c
c   program fakenet
c
c  $Id: fakenet.f,v 1.25 2012/11/25 06:11:07 agnew Exp agnew $
c
c  $Log: fakenet.f,v $
c  Revision 1.25  2012/11/25 06:11:07  agnew
c  added mseed to list of commands, removed unused variable
c
c  Revision 1.24  2012/11/25 06:07:04  agnew
c  fixed error in list of commands
c  added reading of fixed set of modulated annual cycles
c  used separate random-number generator to sample from menus and
c  perturb velocities
c
c  Revision 1.23  2010/04/01 16:34:39  agnew
c  fixed bug for fault segments with no slip
c
c  Revision 1.22  2010/03/26 22:25:09  agnew
c  speed up computation of dislocation effects by looping over fault planes
c  rather than over time
c
c  Revision 1.21  2010/03/25 17:14:39  agnew
c  Added individual sinusoids at stations, randomly selected from a list,
c  and modulated by Brownian bridges (annual only)
c
c  Revision 1.20  2010/03/05 00:47:11  agnew
c  Moved time-series generators  sinus flicker gwhite redwlk to fakesubs
c
c  Revision 1.19  2010/03/03 22:27:07  agnew
c  fixed opening-dislocation case
c
c  Revision 1.18  2009/06/16 00:04:07  agnew
c  fixed format for tst.perf, error in indexing common-mode sinusoids
c
c  Revision 1.17  2009/06/14 04:33:52  agnew
c  expanded number of gaps allowed
c
c  Revision 1.16  2009/06/12 23:05:37  agnew
c  normalization of flknor modified to approximate Langbein's usage
c
c  Revision 1.15  2009/06/12 18:11:03  agnew
c  fixed logfile record of sample rate
c
c  Revision 1.14  2009/06/03 18:00:57  agnew
c  fixed writing of log file to handle cases of no noise;
c  checked random assignment of noises to stations
c
c  Revision 1.13  2009/05/27 22:00:58  agnew
c  Removed subroutines in common with other programs (now in fakesubs.f)
c
c  Revision 1.12  2009/05/27 18:28:12  agnew
c  Added logging of common-mode signals
c  ***Modified logging of rw anf flicker parameters to remove scaling****
c
c  Revision 1.11  2009/05/25 19:52:52  agnew
c  Fixed to respond properly to omitted commands
c  Added common-mode capability, including stochastic noise and multiple
c  sinusoids for each common-mode signal.
c
c  Revision 1.10  2009/03/21 06:24:27  agnew
c  1.9 was used for Phase I. 1.10, fixed bug that wrote out fl noise values
c  as rw noise values
c
c
c  Reads files giving the following information:
c     1. Station codes and locations
c     2. Station velocities (optional)
c     3. Station down times (optional)
c     4. Station noise parameters (optional)
c     5. Station seasonal (cyclical) parameters (optional)
c     6. Network common-mode noise parameters (optional)
c     7. Slip on dislocations (optional)
c
c   and writes out a file giving the time history of the station positions:
c   in the absence of files 2-6, these will be constant.
c
c  maximum number of input lines
      parameter (inmx=500)
c  maximum number of stations
      parameter (maxsta=1000)
c  maximum number of gaps in a series
      parameter (maxgap=500)
c  maximum number of epochs generated
      parameter (maxep=73100)
c  maximum number of fault planes
      parameter (maxpl=5000)
c  maximum number of slip parameters
      parameter (maxsl=500000)
c  maximum number of locations where common-mode signals are produced
      parameter (maxcom=10)
c  maximum number of common-mode sinusoids
      parameter (maxcs=50)
c  maximum number of annual-cycle amplitudes in menu
      parameter (maxann=1000)
      character*1 lcode,dcode,stype
      character*4 code
      double precision time
      double precision begin, end, sample, ctep
      double precision start,stop,gaps
      double precision per
c
c common block for overall parameters
c
c   begin    start time of series (Julian date)
c   end      end time of series (Julian date)
c   sample   sample interval, days
c   geog     latitude and longitude of SW and NE corners of region to be
c               simulated
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for station codes, locations, and velocities
c
c   slat     station latitude
c   slong    station longitude
c   ht       station height
c   code     station 4-char code
c   v        station velocity or other secular trend
c   nsta     number of stations
c   stype    station type of measurement
c              d - displacement (vector is n e u)
c              s - strain (vector is Eee Enn Een)
c              t - tilt (vector is E tilt, N tilt)
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for station up and down times -- all in Julian date
c
c   start(n) start time of data for station n
c   gaps     times of missing data:
c                gaps(n,m,1) last time of good data before the m-th gap at
c                            station n              
c                gaps(n,m,2) first time of good data after the m-th gap at
c                            station n              
c   stop(n)  end time of data for station n
c   ngap(n)  number of data gaps for station n
c
      common/good/start(maxsta),gaps(maxsta,maxgap,2),stop(maxsta),
     1 ngap(maxsta)
c
c common block for station noise parameters
c
c   wh       vector of white noise amplitudes (see stype for vector types)
c   fl       vector of flicker noise amplitudes
c   rw       vector of random-walk noise amplitudes
c   pows     vector of power-law noise amplitudes
c   powe     vector of power-law noise exponents
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)
c
c common block for common-mode noise parameters
c
c   clat     latitude of a common-mode noise source
c   clong    longitude of a common-mode noise source
c   cwh      vector of white noise amplitudes
c   cfl      vector of flicker noise amplitudes
c   crw      vector of random-walk noise amplitudes
c   cpows    vector of power-law noise amplitudes
c   cpowe    vector of power-law noise exponents
c   ncomm    number of common-mode sources
c
      common /cmnoi/clat(maxcom),clong(maxcom),
     1  cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),ncomm
c
c common block for common-mode sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  ncsp(maxcom),numcs
c
c common block for common-mode noise series
c
      common /cmser/cmseries(maxep,maxcom,3)
c
c common block for individual station sinusoids
c
c   annual   vector of annual cycle amplitudes
c   as       vector of amplitude modulation
c   ps       vector of phase modulation
c   nann     flag for presence of annual cycles
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 lst(maxpl),lse(maxpl),nflt
c
c common block for fault slips
c
      common/slips/time(maxsl),rake(maxsl),slip(maxsl),nslp
c
c
c common blocks of scanrp for handling commands:
c
      character*80 input(inmx)
      common /store/ input
      common /ndict/ iecho,nin
c-----------------------------------------------------------------------------
c                       Program starts here
c-----------------------------------------------------------------------------
c
c Read commands; completed when  exec  command is read
c
      call scanrp
c
c Initialize random number generator, and create default output name
c
      call rangod
c
c Read station names and positions
c
      call rdsta1
c
c Read station velocities
c
      call rdsta2
c
c Read station performance
c
      call rdsta3
c
c Read station noise parameters
c
      call rdsta4
c
c Produce randomized station noise parameters
c
      call rdsta5
c
c Produce station annual-cycle amplitudes
c
      call rdsta6
c
c Produce randomized station annual-cycle amplitudes
c
      call rdsta7
c
c Read fault slip information
c
      call rdflt
c
c Read common-mode noise parameters
c
      call rdcommn
c
c Write a log file of all parameters
c
      call logit
c
c Compute and write out the results
c
      call fakemake
c
      stop
      end
c=======================================================================
      blockdata initall
c
c initial values for material that might be written to the output file
c
c  maximum number of stations
      parameter (maxsta=1000)
      parameter (maxsta3=3000)
c  maximum number of gaps in a series
      parameter (maxgap=500)
      parameter (maxgap2=1000000)
c  maximum number of fault planes
      parameter (maxpl=5000)
c  maximum number of slip parameters
      parameter (maxsl=500000)
c  maximum number of locations where common-mode signals are produced
      parameter (maxcom=10)
      parameter (maxcom3=30)
c  maximum number of common-mode sinusoids
      parameter (maxcs=50)
      parameter (maxcs6=300)
c
      character*1 lcode,dcode,stype
      character*4 code
      double precision time
      double precision begin, end, sample, ctep
      double precision start,stop,gaps
      double precision per
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for station up and down times -- all in Julian date
c
      common/good/start(maxsta),gaps(maxsta,maxgap,2),stop(maxsta),
     1 ngap(maxsta)
c
c common block for station noise parameters
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)

c
c common block for common-mode noise parameters
c
      common /cmnoi/clat(maxcom),clong(maxcom),
     1  cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),ncomm
c
c common block for common-mode sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  ncsp(maxcom),numcs
c
c common block for individual station sinusoids
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann

c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 lst(maxpl),lse(maxpl),nflt
c
c
c common block for fault slips
c
      common/slips/time(maxsl),rake(maxsl),slip(maxsl),nslp
c
      data begin/0.0d0/,end/0.0d0/,sample/0.0d0/,geog/4*0.0/
c
      data slat/maxsta*99.0/,slong/maxsta*370.0/,ht/maxsta*0.0/,
     1  code/maxsta*'    '/,v/maxsta3*0.0/,nsta/0/,stype/maxsta*' '/
c
      data start/maxsta*0.0d0/,gaps/maxgap2*0.0d0/,
     1      stop/maxsta*0.0d0/,ngap/maxsta*0/
c
      data wh/maxsta3*0.0/,fl/maxsta3*0.0/,rw/maxsta3*0.0/,
     1  pows/maxsta3*0.0/,powe/maxsta3*0.0/
c
      data clat/maxcom*99.0/,clong/maxcom*370.0/,cwh/maxcom3*0.0/,
     1 crw/maxcom3*0.0/,cpows/maxcom3*0.0/,cpowe/maxcom3*0.0/,ncomm/0/
c
      data ctep/maxcs*0.d0/,csin/maxcs6*0.0/,per/maxcom*0.d0/,
     1  ncsp/maxcom*0/,numcs/0/
c
      data annual/maxsta3*0.0/,as/maxsta3*0.0/,ps/maxsta3*0.0/,nann/0/
c
      data gt1/maxpl*0.0/,gg1/maxpl*0.0/,gt2/maxpl*0.0/,gg2/maxpl*0.0/,
     1 dep/maxpl*0.0/,wid/maxpl*0.0/,dip/maxpl*0.0/,nflt/0/
c
      data time/maxsl*0.d0/ rake/maxsl*0.0/,slip/maxsl*0.0/,nslp/0/
c
      end
c=======================================================================
      subroutine rangod
      character*15 dnam
      integer time
      double precision begin, end, sample
      dimension ia(9)
c
c common block for unique name (date string)
c
      common/defnam/dnam
c
c common block for random-number seed
c
      common/rseed/iseed,mseed
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for noise normalizations
c
      common/nornoise/flknor,rdwnor
c
c initialize main random-number generator
c
      call getint('seed',iseed,iok)
      if(iok.le.0) iseed=12345
      call init_genrand(is)
c
c initialize secondary random-number generator
c
      call getint('msee',mseed,iok)
      if(iok.le.0) mseed=12345
c
c default output name (date-specific string)
c
      it=time()
      call gmtime(it,ia)
      iy=ia(6)+1900
      id=ia(8)+1
      n=ia(1)+100*(ia(2)+100*ia(3))
      write(dnam(1:15),100) iy,id,n
 100  format(i4,'.',i3.3,'.',i6.6)
c
c get sample interval
c
      call getone('samp',tmp,iok)
      sample=tmp
      if (iok.le.0) then
        write(6,102)
 102    format(' Sample interval needed (in days) ')
        stop
      endif
c
c normalizations for flicker and rw noise
c    (factor of two in flknor added Fri Jun 12 16:05:01 PDT 2009)
c
      rdwnor=sqrt(sample/365.25)
      flknor=2*sqrt(rdwnor)
      return
      end
c=======================================================================
      subroutine rdsta1
c
c$$$$ calls getarr,getchr
c
c  Reads in station location information, keeping only stations that fall
c within spatial limits. File is assumed to have the format
c   code lat long height
c where the lat and long are geographic coordinates, in decimal degrees
c (East positive) and the height c is in m
c
      parameter (maxsta=1000)
      character*80 name
      character*4 code,ctmp
      character*1 stype
      double precision begin, end, sample
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
      data lui/11/
      nsta=1
c
c get geographic limits
c
      call getarr('area',geog,4,nf)
      if(nf.eq.4) then
        if(geog(3).gt.180) geog(3)=geog(3)-360
        if(geog(4).gt.180) geog(4)=geog(4)-360
      endif
      if(nf.le.0) then
        geog(1)=-90
        geog(2)=90
        geog(3)=-180
        geog(4)=180
      endif
c
c get input filename, and read the data
c
      call getchr('loca',name,iok)
      if (iok.le.0) then
        write(6,100)
 100    format(' Input file name needed (location)')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,*,end=7) ctmp,tlat,tlong,tht
      if(tlong.gt.180) tlong=tlong-360
      if(tlat.ge.geog(1).and.tlat.le.geog(2).and.
     2 tlong.ge.geog(3).and.tlong.le.geog(4)) then
         code(nsta)=ctmp
         slat(nsta)=tlat
         slong(nsta)=tlong
         ht(nsta)=tht
         nsta=nsta+1
      endif
      if(nsta.le.maxsta) go to 5
      write(6,102) maxsta
 102  format(' Only ',i6, ' station locations read in')
 7    nsta=nsta-1
      close(lui)
      return
      end
c=======================================================================
      subroutine rdsta2
c
c$$$$ calls getarr,getchr
c
c  Reads in station velocity information, perturbs velocities by random
c   amounts if this command is given
c
      parameter (maxsta=1000)
      character*80 name
      character*4 code,ctmp
      character*1 stype
      dimension ifnd(maxsta),verr(3)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
      data lui/11/
      data ifnd/maxsta*0/
c
c get input filename, and read the data
c
      call getchr('velo',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (velocities)')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,*,end=7) ctmp,tvn,tve,tvu
      iok=0
      do i=1,nsta
        if(ctmp.eq.code(i)) then
          v(i,1)=tvn
          v(i,2)=tve
          v(i,3)=tvu
          iok=1
          ifnd(i)=1
        endif
      enddo
      if(iok.eq.0) write(6,102) ctmp
 102  format('No location for velocities for station ',a4)
      go to 5
 7    do i=1,nsta
        if(ifnd(i).eq.0) write(6,104) code(i)
 104    format('No velocities for station ',a4)
      enddo
      close(lui)
c
c perturb the velocities by random amounts, scaling gaussians by the random
c levels given by the verr command
c
      call getarr('verr',verr,3,nt)
      if(nt.eq.3) then
        do i=1,nsta
          do k=1,3
            v(i,k)=v(i,k) + verr(k)*gauss()
          enddo
        enddo
      endif
      return
      end
c=======================================================================
      subroutine rdsta3
c
c$$$$ calls getarr,getchr,getone,juldat,toymd
c
c  Reads in network performance information
c
      parameter (maxsta=1000)
      parameter (maxgap=500)
      character*80 name,line
      character*4 code,ctmp
      character*1 type
      character*1 stype
      double precision begin, end, sample
      double precision start,stop,gaps
      double precision gap1,gap2
      dimension it(5),rit(5),itm(6)
      dimension ifnd(maxsta)
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for station up and down time
c
      common/good/start(maxsta),gaps(maxsta,maxgap,2),stop(maxsta),
     1 ngap(maxsta)
      data ifnd/maxsta*0/
c
      data lui/11/
c
c get temporal limits--which are converted to Julian date
c
      call getarr('begi',rit,5,nt)
      if(nt.le.0) then
        write(6,100)
 100    format(' Begin time needed ')
        stop
      endif
      do i=1,5
        it(i)=0
      enddo
      do i=1,nt
        it(i)=rit(i)
      enddo
      call toymd(it,itm)
      begin=dat2jul(itm)
      call getarr('end ',rit,5,nt)
      if(nt.le.0) then
        write(6,102)
 102    format(' End time needed ')
        stop
      endif
      do i=1,5
        it(i)=0
      enddo
      do i=1,nt
        it(i)=rit(i)
      enddo
      call toymd(it,itm)
      end=dat2jul(itm)
      if(begin.ge.end) then
        write(6,104)
 104    format('Begin time must precede end time')
        stop
      endif
c
c get input filename
c
      call getchr('perf',name,iok)
c
c if no performance file, show stations as always on
c
      if (iok.lt.0) then
        do i=1,nsta
          start(i)=begin
          stop(i)=end
          ngap(i)=0
        enddo
        return
      endif
      if (iok.eq.0) then
        write(6,106)
 106    format(' Input file name needed (performance)')
        stop
      endif
      open(lui,file=name,status='old')
c
c read performance file
c
 5    read(lui,120,end=7) line(1:80)
 120  format(a80)
      if(line(1:1).eq."s") read(line,*) type, ctmp, iy,id
      if(line(1:1).eq."m") read(line,*) type, ctmp, iy,id,miss
      if(line(1:1).eq."e") read(line,*) type, ctmp, iy,id
      do i=1,5
        it(i)=0
      enddo
      it(1)=iy
      it(2)=id
      call toymd(it,itm)
      iok=0
      do i=1,nsta
        if(ctmp.eq.code(i)) then
           ifnd(i)=1
           if(type.eq."s".or.type.eq."m".or.type.eq."e") iok=1
           if(type.eq."s") start(i)=dat2jul(itm)
           if(type.eq."e") stop(i)=dat2jul(itm)
           if(type.eq."m") then
                 gap1=dat2jul(itm)
                 gap2=dat2jul(itm)+miss+1
                 if(gap1.ge.begin.and.gap1.le.end.and.
     1              gap2.ge.begin.and.gap2.le.end)    then
                    ngap(i)=ngap(i)+1
                    gaps(i,ngap(i),1)=gap1
                    gaps(i,ngap(i),2)=gap2
                 endif
                 if(gap1.lt.begin.and.
     1              gap2.ge.begin.and.gap2.le.end)    then
                    ngap(i)=ngap(i)+1
                    gaps(i,ngap(i),1)=begin
                    gaps(i,ngap(i),2)=gap2
                 endif
                 if(gap1.ge.begin.and.gap1.le.end.and.
     1                                  gap2.gt.end)    then
                    ngap(i)=ngap(i)+1
                    gaps(i,ngap(i),1)=gap1
                    gaps(i,ngap(i),2)=end
                 endif
           endif
        endif
      enddo
      if(iok.eq.0) write(6,108) ctmp
 108  format('Performance info for station ',a4,' has no match')
      go to 5
 7    continue
      close(lui)
      do i=1,nsta
        if(ifnd(i).eq.0) write(6,110) code(i)
 110    format('No start-stop info for station ',a4)
      enddo
      return
      end
c=======================================================================
      subroutine rdsta4
c
c$$$$ calls getchr
c
c  Reads in station noise information
c
      parameter (maxsta=1000)
      character*80 name,line
      character*4 code,ctmp
      character*1 type,c
      character*1 stype
      dimension p(3),pe(3),ifnd(maxsta)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c     
c common block for station noise parameters
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)
c
c common block for noise normalizations
c
      common/nornoise/flknor,rdwnor
c
      data lui/11/
      data ifnd/maxsta*0/
c
c get input filename, and read the data
c
      call getchr('stan',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (noise)')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,106,end=7) line(1:80)
 106  format(a80)
      iok=0
      do k=1,3
        p(k)=0
      enddo
      c=line(1:1)
      if(c.eq."w".or.c.eq."r".or.c.eq."f") then
        read(line,*) type, ctmp, (p(i),i=1,3)
        do i=1,nsta
          if(ctmp.eq.code(i)) then
            iok=1
            ifnd(i)=1
            if(c.eq."w") then
              do k=1,3
               wh(i,k)=p(k)
              enddo
            endif
            if(c.eq."f") then
              do k=1,3
               fl(i,k)=flknor*p(k)
              enddo
            endif
            if(c.eq."r") then
              do k=1,3
               rw(i,k)=rdwnor*p(k)
              enddo
            endif
          endif
        enddo
      endif
      if(c.eq."p") then
        read(line,*) type, ctmp, (p(i),pe(i),i=1,3)
        do i=1,nsta
          if(ctmp.eq.code(i)) then
            do k=1,3
              pows(i,k)=p(k)
              powe(i,k)=pe(k)
            enddo
          endif
        enddo
      endif
      if(iok.eq.0) write(6,108) ctmp
 108  format('Noise info for station ',a4,' has no match')
      go to 5
 7    do i=1,nsta
        if(ifnd(i).eq.0) write(6,110) code(i)
 110    format('No noise information for station ',a4)
      enddo
      close(lui)
      return
      end
c=======================================================================
      subroutine rdsta5
c
c$$$$ calls getchr
c
c  Reads in an array of noise information, then selects from it (randomly,
c   with replacement) to get the actual noise parameters that will be used.
c
      parameter (maxsta=1000)
      parameter (maxnoi=1000)
      character*80 name,line
      character*4 code
      character*1 type,c
      character*1 stype
      dimension p(3),pe(3)
C      dimension it(5),itm(6)
      dimension swh(maxnoi,3),sfl(maxnoi,3),srw(maxnoi,3),
     1   spows(maxnoi,3),spowe(maxnoi,3)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c     
c common block for station noise parameters
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)
c
c common block for noise normalizations
c
      common/nornoise/flknor,rdwnor
c
      data lui/11/
c
c get input filename, and read the data
c
      call getchr('samn',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (sample noise)')
        stop
      endif
      nwh=0
      nfl=0
      nrw=0
      npow=0
      open(lui,file=name,status='old')
 5    read(lui,102,end=7) line(1:80)
 102  format(a80)
      do k=1,3
        p(k)=0
      enddo
      c=line(1:1)
      if(c.eq."w".or.c.eq."r".or.c.eq."f") then
        read(line,*) type, (p(i),i=1,3)
        if(c.eq."w") then
          nwh=nwh+1
          do k=1,3
           swh(nwh,k)=p(k)
          enddo
        endif
        if(c.eq."f") then
          nfl=nfl+1
          do k=1,3
           sfl(nfl,k)=p(k)*flknor
          enddo
        endif
        if(c.eq."r") then
          nrw=nrw+1
          do k=1,3
           srw(nrw,k)=p(k)*rdwnor
          enddo
        endif
      endif
      if(c.eq."p") then
        npow=npow+1
        read(line,*) type, (p(i),pe(i),i=1,3)
        do k=1,3
         spows(npow,k)=p(k)
         spowe(npow,k)=pe(k)
        enddo
      endif
      go to 5
 7    close(lui)
c
c   now do the random assignment: select values randomly from set
c   read in, separately for each component, c  for white, flicker,
c   random-walk, and power-law noise
c
      do i=1,nsta
        do j=1,3
          k=int(nwh*bran())+1
          if(nwh.gt.0) wh(i,j)=swh(k,j)
        enddo
        do j=1,3
          k=int(nfl*bran())+1
          if(nfl.gt.0) fl(i,j)=sfl(k,j)
        enddo
        do j=1,3
          k=int(nrw*bran())+1
          if(nrw.gt.0) rw(i,j)=srw(k,j)
        enddo
        do j=1,3
          k=int(npow*bran())+1
          if(npow.gt.0) pows(i,j)=spows(k,j)
          if(npow.gt.0) powe(i,j)=spowe(k,j)
        enddo
      enddo
      return
      end
c=======================================================================
      subroutine rdsta6
c
c$$$$ calls getchr
c
c  Reads in an array of annual-cycle amplitudes, and modulations
c
      parameter (maxsta=1000)
      character*80 name,line
      character*4 code
      character*4 ctmp
      character*1 stype
      dimension p(9)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for individual station sinusoids
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann
c
      data lui/11/
c
c get input filename, and read the data
c
      call getchr('annu',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (annual amps and modulations)')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,102,end=7) line(1:80)
 102  format(a80)
      read(line,*) ctmp, (p(i),i=1,9)
      do i=1,nsta
        if(ctmp.eq.code(i)) then
          nann=nann+1
          annual(i,1)=p(1)
          as(i,1)=p(2)
          ps(i,1)=p(3)
          annual(i,2)=p(4)
          as(i,2)=p(5)
          ps(i,2)=p(6)
          annual(i,3)=p(7)
          as(i,3)=p(8)
          ps(i,3)=p(9)
        endif
      enddo
      go to 5
 7    close(lui)
      return
      end
c=======================================================================
      subroutine rdsta7
c
c$$$$ calls getchr
c
c  Reads in an array of annual-cycle amplitudes, then selects from it (randomly,
c   with replacement) to get the actual amplitudes that will be used.
c
      parameter (maxsta=1000)
      parameter (maxann=1000)
      character*80 name,line
      character*4 code
      character*1 stype
      dimension p(2)
      dimension annmen(maxann,3)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for individual station sinusoids
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann
c
      data lui/11/
      data ascl/0.0/pscl/0.0/nannl/0/
c
c get input filename, and read the data
c
      call getchr('rann',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (annual amplitudes)')
        stop
      endif
      nann=1
      open(lui,file=name,status='old')
 5    read(lui,102,end=7) line(1:80)
 102  format(a80)
      read(line,*) (annmen(nannl+1,i),i=1,3)
      nannl=nannl+1
      go to 5
 7    close(lui)
c
c   get the maximum amounts by which to modulate the annual cycles
c
      call getarr('modu',p,2,iok)
      if (iok.lt.0) return
      if (iok.ne.2) then
        write(6,104)
 104    format(' Modulation amounts needed ')
        stop
      endif
      ascl=p(1)
      pscl=p(2)
c
c   select randomly from the amplitudes, with the modulations being
c    uniformly distributed from zero to the value given--but rounded
c    to two decimals, since that is how they are written out
c
      if(nannl.gt.0) then
        do i=1,nsta
          nann=nann+1
          k=int(nannl*bran())+1
          do j=1,3
            annual(i,j)=annmen(k,j)
            as(i,j)=nint(1000*ascl*bran())/1000.
            ps(i,j)=nint(1000*pscl*bran())/1000.
          enddo
        enddo
      endif
      return
      end
c=======================================================================
      subroutine rdflt
c
c$$$$ calls getarr,getchr,getone
c
c  Reads in fault plane and slip information
c
      parameter (maxpl=5000)
      parameter (maxsl=500000)
      character*80 name,line
      character*1 type
      character*1 lcode,dcode
      double precision time,dat2jul
      save ifs
      dimension it(5),itm(6)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 lst(maxpl),lse(maxpl),nflt
c
c common block for fault slips
c
      common/slips/time(maxsl),rake(maxsl),slip(maxsl),nslp
c
      data lui/11/
      data dr/.0174533/
      data ifs/0/
c
c get input filename, and read the data
c
      call getchr('faul',name,iok)
      if (iok.eq.-99) return
      if (iok.eq.0) then
        write(6,102)
 102    format(' Input file name needed (fault geometry and slip)')
        stop
      endif
      open(lui,file=name,status='old')
 5    read(lui,104,end=7) line(1:80)
 104  format(a80)
c
c data read in as
c  
c g lat1 long1 lat2 long2 depcode depth width dip
c
      if(line(1:1).eq."g") then
         nflt=nflt+1
         read(line,*) type,gt1(nflt),gg1(nflt),gt2(nflt),gg2(nflt),
     1     lcode(nflt),dcode(nflt),dep(nflt),wid(nflt),dip(nflt)
         if(dcode(nflt).eq.'t') dep(nflt) = dep(nflt) +
     1           0.5*wid(nflt)*sin(dr*dip(nflt))
         if(nflt.gt.1.and.lst(nflt-1).ne.0) lse(nflt-1)=nslp
         ifs=1
         lst(nflt)=0
         lse(nflt)=0
      endif
c
c data read in as
c  
c s slip rake year day hour
c
      if(line(1:1).eq."s") then
        nslp=nslp+1
        if(ifs.eq.1) then
          lst(nflt)=nslp
          ifs=0
        endif
        do i=1,6
          it(i)=0
        enddo
        read(line,*) type,slip(nslp),rake(nslp),(it(i),i=1,3)
        call toymd(it,itm)
        time(nslp) = dat2jul(itm)
      endif
      go to 5
 7    continue
      if(lst(nflt).gt.0) lse(nflt)=nslp
      close(lui)
      return
      end
c=======================================================================
      subroutine rdcommn
c
c$$$$ calls getchr
c
c  Reads in noise information for common-mode series, and computes them
c
      parameter (maxep=73100)
      parameter (maxcom=10)
      parameter (maxcs=50)
      character*80 name,line
      character*1 type,c
      double precision begin, end, sample, ctep
      double precision per,dr
      dimension aux(maxep)
      dimension p(3),pe(3)
      dimension it(5),itm(6)
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for noise normalizations
c
      common/nornoise/flknor,rdwnor
c
c common block for common-mode noise parameters
c
      common /cmnoi/clat(maxcom),clong(maxcom),
     1  cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),ncomm
c
c common block for common-mode sinusoids; ctep is the epoch time,
c csin the sine and cosine amplitudes, and per the periods, for each
c of the three components, ncsp the number of sinusoids for each
c common-mode series, and numcs the total number of sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  ncsp(maxcom),numcs
c
c common block for common-mode noise series
c
      common /cmser/cmseries(maxep,maxcom,3)
c
      data lui/11/
      data dr/0.01745329252d0/
c
c get input filename, and read the data
c
      call getchr('comm',name,iok)
      if (iok.lt.0) return
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed (common-mode noise)')
        stop
      endif
      ncomm=0
      numcs=0
      open(lui,file=name,status='old')
 5    read(lui,102,end=7) line(1:80)
 102  format(a80)
      iok=0
      do k=1,3
        p(k)=0
      enddo
      c=line(1:1)
c
c  if first letter is l, have a new location
c
      if(c.eq."l") then
        ncomm=ncomm+1
        ncsp(ncomm)=0
        read(line,*) type, clat(ncomm),clong(ncomm)
      endif
      if(c.eq."w".or.c.eq."r".or.c.eq."f") then
        read(line,*) type, (p(i),i=1,3)
        if(c.eq."w") then
          do k=1,3
            cwh(ncomm,k)=p(k)
          enddo
        endif
        if(c.eq."f") then
          do k=1,3
           cfl(ncomm,k)=flknor*p(k)
          enddo
        endif
        if(c.eq."r") then
          do k=1,3
           crw(ncomm,k)=rdwnor*p(k)
          enddo
        endif
      endif
      if(c.eq."p") then
        read(line,*) type, (p(i),pe(i),i=1,3)
        do k=1,3
         cpows(ncomm,k)=p(k)
         cpowe(ncomm,k)=pe(k)
        enddo
      endif
c
c Read parameters for sinusoidal behavior: the phases are assumed to be
c  relative to the time given as y-d-h-m-s at the end, and the period
c  is in days. The amps and phases are converted to cos and sin parts at
c  the begin time
c
c The  ncsp  array contains, for each entry, the number of sinusoids for
c   a given common-mode signal.
c
      if(c.eq."s") then
        numcs=numcs+1
        ncsp(ncomm)=ncsp(ncomm)+1
        read(line,*) type,(pe(i),i=1,3),(p(i),i=1,3),per(numcs),
     1       (it(i),i=1,5)
        call toymd(it,itm)
        ctep(numcs) = dat2jul(itm)
        do k=1,3
          ph=dr*(p(k) + 360*((ctep(numcs)-begin)/per(numcs)))
          csin(numcs,k,1)=pe(k)*cos(ph)
          csin(numcs,k,2)=pe(k)*sin(ph)
        enddo
      endif
      go to 5
 7    close(lui)
c
c now construct the series, which have to cover the entire span
c
      ne=nint((end-begin)/sample)+1
      if(ne.gt.maxep) then
        write(6,106) ne,maxep
 106    format(i7,' times in time series, more than ',i6,' allowed')
        stop
      endif
      do i=1,ncomm
	do j=1,3
          if(cwh(i,j).gt.0) call gwhite(aux,ne)
          do k=1,ne
            cmseries(k,i,j)=cmseries(k,i,j)+aux(k)*cwh(i,j)
          enddo
          if(cfl(i,j).gt.0) call flicker(aux,ne)
          do k=1,ne
            cmseries(k,i,j)=cmseries(k,i,j)+aux(k)*cfl(i,j)
          enddo
          if(crw(i,j).gt.0) call redwlk(aux,ne)
          do k=1,ne
            cmseries(k,i,j)=cmseries(k,i,j)+aux(k)*crw(i,j)
          enddo
c
c for sinusoids, have to take variable numbers for each common-mode
c   point
c
          if(ncsp(i).ne.0) then
            inh=0
            do kk=1,i
              inl=inh+1
              inh=inl+ncsp(kk)-1
            enddo
            do ii=inl,inh
              call sinus(aux,csin(ii,j,1),csin(ii,j,2),per(ii),ne)
              do k=1,ne
                cmseries(k,i,j)=cmseries(k,i,j)+aux(k)
              enddo
            enddo
          endif
        enddo
      enddo
      return
      end
c=======================================================================
      subroutine logit
c writes out all parameters used
      parameter (maxsta=1000)
      parameter (maxgap=500)
      parameter (maxep=73100)
      parameter (maxpl=5000)
      parameter (maxsl=500000)
      parameter (maxcom=10)
      parameter (maxcs=50)
      character*1 stype
      character*1 lcode,dcode
      character*4 code
      character*15 dnam
      character*80 name
      double precision stme
      double precision begin, end, sample, ctep
      double precision start,stop,gaps
      double precision per,dr
      dimension it(5),itm(6)
      dimension a(3),p(3)
c
c common blocks for default output name, seed
c
      common/defnam/dnam
      common/rseed/iseed,mseed
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for noise normalizations
c
      common/nornoise/flknor,rdwnor
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for station up and down time
c
      common/good/start(maxsta),gaps(maxsta,maxgap,2),stop(maxsta),
     1 ngap(maxsta)
c
c common block for station noise parameters
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 lst(maxpl),lse(maxpl),nflt
c
c common block for fault slips
c
      common/slips/stme(maxsl),rake(maxsl),slip(maxsl),nslp
c
c common block for common-mode noise parameters
c
      common /cmnoi/clat(maxcom),clong(maxcom),
     1  cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),ncomm
c
c common block for common-mode sinusoids
c
      common /cmsins/ctep(maxcs),csin(maxcs,3,2),per(maxcom),
     1  ncsp(maxcom),numcs
c
c common block for individual station sinusoids
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann
c
c Get output filename and open file
c
      data luo/12/
      data dr/0.01745329252d0/
      call getchr('logf',name,iok)
      if(iok.lt.0) return
c
c default name is given by time.log
c
      if(iok.lt.4) then
        name(1:15)=dnam
        name(16:19)=".log"
      endif
      if(iok.ge.0) then
        open(luo,file=name,status="unknown")
        write(luo,101)
 101    format("cat << XXX > tst.coords")
        do i=1,nsta
           write(luo,102) code(i),slat(i),slong(i),ht(i)
 102       format(a4,f10.5,f11.5,f9.2)
        enddo
        write(luo,103)
 103    format("XXX")
        write(luo,104)
 104    format("cat << XXX > tst.vels")
        do i=1,nsta
           write(luo,106) code(i),(v(i,k),k=1,3)
 106       format(a4,3f10.3)
        enddo
        write(luo,103)
        write(luo,108)
 108    format("cat << XXX > tst.perf")
        do i=1,nsta
           call jul2dat(start(i),itm)
           call toyd(itm,it)
           write(luo,110) "s",code(i),it
 110       format(a1,1x,a4,i5,i4,3i2)
           do j=1,ngap(i)
              call jul2dat(gaps(i,j,1),itm)
              call toyd(itm,it)
              write(luo,112) code(i),it(1),it(2),
     1                     nint(gaps(i,j,2)-gaps(i,j,1))
 112          format("m ",a4,i5,i4,i6)
           enddo
           call jul2dat(stop(i),itm)
           call toyd(itm,it)
           write(luo,110) "e",code(i),it
        enddo
        write(luo,103)
c
c  write out noise values; note reverse scaling (changed from unscaled
c    27 May 2009)
c
        nn=0
        do i=1,nsta
         if(wh(i,1).ne.0.or.wh(i,2).ne.0.or.wh(i,3).ne.0) nn=nn+1
         if(fl(i,1).ne.0.or.fl(i,2).ne.0.or.fl(i,3).ne.0) nn=nn+1
         if(rw(i,1).ne.0.or.rw(i,2).ne.0.or.rw(i,3).ne.0) nn=nn+1
         if(pows(i,1).ne.0.or.pows(i,2).ne.0.or.pows(i,3).ne.0)
     1     nn=nn+1
        enddo
        if(nn.gt.0) then
         write(luo,114)
 114     format("cat << XXX > tst.noise")
         do i=1,nsta
           if(wh(i,1).ne.0.or.wh(i,2).ne.0.or.wh(i,3).ne.0)
     1        write(luo,116) "w",code(i),(wh(i,k),k=1,3)
 116       format(a1,1x,a4,3f10.4)
           if(fl(i,1).ne.0.or.fl(i,2).ne.0.or.fl(i,3).ne.0)
     1        write(luo,116) "f",code(i),(fl(i,k)/flknor,k=1,3)
           if(rw(i,1).ne.0.or.rw(i,2).ne.0.or.rw(i,3).ne.0)
     1        write(luo,116) "r",code(i),(rw(i,k)/rdwnor,k=1,3)
           if(pows(i,1).ne.0.or.pows(i,2).ne.0.or.pows(i,3).ne.0)
     1        write(luo,118) "p",code(i),(pows(i,k),powe(i,k),k=1,3)
 118       format(a1,1x,a4,6f10.4)
         enddo
         write(luo,103)
        endif
        if(nslp.gt.0) then
         write(luo,120)
 120     format("cat << XXX > tst.slip")
         do i=1,nflt
c correct depths to be top of plane
           d=dep(i)
           if(dcode(i).eq."t") d = d -
     1           0.5*wid(i)*sin(dr*dip(i))
           write(luo,122) gt1(i),gg1(i),gt2(i),gg2(i),lcode(i),dcode(i),
     1                   d,wid(i),dip(i)
 122       format("g",2(f9.4,f10.4),1x,a1,1x,a1,2f8.3,f5.1)
           do j=lst(i),lse(i)
             call jul2dat(stme(j),itm)
             call toyd(itm,it)
             write(luo,124) slip(j),rake(j),it
 124         format("s",f10.4,f6.1,i5,i4,3i3)
           enddo
         enddo
         write(luo,103)
        endif
        if(ncomm.gt.0) then
         write(luo,126)
 126     format("cat << XXX > tst.commod")
         do i=1,ncomm
           write(luo,128) "l",clat(i),clong(i)
 128       format(a1,1x,f9.5,f12.5)
           if(cwh(i,1).ne.0.or.cwh(i,2).ne.0.or.cwh(i,3).ne.0)
     1       write(luo,130) "w",(cwh(i,k),k=1,3)
 130       format(a1,1x,3f10.4)
           if(cfl(i,1).ne.0.or.cfl(i,2).ne.0.or.cfl(i,3).ne.0)
     1       write(luo,130) "f",(cfl(i,k)/flknor,k=1,3)
           if(crw(i,1).ne.0.or.crw(i,2).ne.0.or.crw(i,3).ne.0)
     1       write(luo,130) "r",(crw(i,k)/rdwnor,k=1,3)
           if(cpows(i,1).ne.0.or.cpows(i,2).ne.0.or.cpows(i,3).ne.0)
     1       write(luo,132) "p",(cpows(i,k),cpowe(i,k),k=1,3)
 132       format(a1,1x,6f10.4)
c
c see if there is a common-mode sinusoid; if so, find the sinusoids that
c  are appropriate, and convert back to amp and phase
c
           if(ncsp(i).ne.0) then
             inh=0
             do kk=1,i
               inl=inh+1
               inh=inl+ncsp(kk)-1
             enddo
             do ii=inl,inh
               call jul2dat(ctep(ii),itm)
               call toyd(itm,it)
               do k=1,3
                 a(k)=sqrt(csin(ii,k,1)**2+csin(ii,k,2)**2)
                 p(k)=0.
                 if(a(k).gt.0) then
                   p(k)=atan2(csin(ii,k,2),csin(ii,k,1))/dr -
     1                     360*(ctep(ii)-begin)/per(ii)
                   p(k)=mod(p(k),360)
                   if(p(k).lt.-180) p(k)=p(k)+360
                   if(p(k).gt.180) p(k)=p(k)-360
                 endif
               enddo
               write(luo,134) "s",(a(k),k=1,3),(p(k),k=1,3),per(ii),it
 134           format(a1,1x,3f8.4,3f7.1,f10.4,1x,5i4)
             enddo
           endif
         enddo
         write(luo,103)
        endif
        if(nann.gt.0) then
          write(luo,136)
 136      format("cat << XXX > tst.annual")
          do i=1,nsta
             write(luo,138) code(i),
     1                      (annual(i,k),as(i,k),ps(i,k),k=1,3)
 138         format(a4,3(f10.3,f7.3,f7.3))
          enddo
          write(luo,103)
        endif
        write(luo,166)
 166    format("cat << XXX > tst.cmmds")
        call jul2dat(begin,itm)
        call toyd(itm,it)
        write(luo,168) iseed,it
 168    format("seed",i9,/,"begin",i5,i4,3i3)
        call jul2dat(end,itm)
        call toyd(itm,it)
        write(luo,170) it,sample
 170    format("end ",i5,i4,3i3,/,"samp",f10.6)
        write(luo,172) (geog(i),i=1,4)
 172    format("area",2f11.4,2f12.4)
        write(luo,174)
 174    format("loca tst.coords",/,"perf tst.perf",/,
       1 "velo tst.vels")
        if(nslp.gt.0) write(luo,175)
 175    format("faul tst.slip")
        if(nn.gt.0) write(luo,176)
 176    format("stan tst.noise")
        if(ncomm.gt.0) write(luo,178)
 178    format("comm tst.commod")
        if(nann.gt.0) write(luo,180)
 180    format("annu tst.annual")
        write(luo,200)
 200    format("exec",/,"XXX")
        close(luo)
      endif
      return
      end
c=======================================================================
      subroutine fakemake
c
      parameter (maxsta=1000)
      parameter (maxgap=500)
      parameter (maxep=73100)
      character*1 stype
      character*4 code
      character*15 dnam
      character*80 name
      double precision time
      double precision begin, end, sample
      double precision start,stop,gaps
      dimension x(maxep,3)
c     dimension err(maxep,3)
      dimension aux1(maxep),aux2(maxep),aux3(maxep)
      dimension itm(6),it(5)
c
c common block for default output name
c
      common/defnam/dnam
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for station codes, locations, and velocities
c
      common/stas/slat(maxsta),slong(maxsta),ht(maxsta),code(maxsta),
     1            v(maxsta,3),nsta,stype(maxsta)
c
c common block for station up and down time
c
      common/good/start(maxsta),gaps(maxsta,maxgap,2),stop(maxsta),
     1 ngap(maxsta)
c
c common block for station noise parameters
c
      common /noise/wh(maxsta,3),fl(maxsta,3),rw(maxsta,3),
     1            pows(maxsta,3),powe(maxsta,3)
c
c common block for individual station sinusoids
c
      common /cycle/annual(maxsta,3),as(maxsta,3),ps(maxsta,3),nann
c
      data luo/12/
c
c Get output filename and open file
c
c     call getchr('outp',name,iok)
c     if(iok.lt.4) then
c     endif
c
      ne=nint((end-begin)/sample)+1
      if(ne.gt.maxep) then
        write(6,100) ne,maxep
 100    format(i7,' times in time series, more than ',i6,' allowed')
        stop
      endif
c     construct the time series for each station
c
      do i=1,nsta
        if(stop(i).lt.begin.or.start(i).gt.end) go to 13
        name(1:4)=code(i)
        name(5:5)='.'
        name(6:20)=dnam
        name(21:24)='.csv'
        open(luo,file=name,status='unknown')
	write(luo,102) code(i),slat(i),slong(i),ht(i)
 102    format('Station ',a4,',',f11.4,',',f12.4,',',f10.2)
        in1=1
        if(start(i).gt.begin) in1=nint((start(i)-begin)/sample)+1
c
c velocity--offset so that position will be zero at start of data
c  also correct for rates being per year, while times are in days
c
        do k=1,ne
          time=begin + (k-1)*sample
          do m=1,3
            if(time.lt.start(i)) x(k,m)=0
            if(time.ge.start(i).and.start(i).ge.begin)
     1              x(k,m)=v(i,m)*(time-start(i))/(sample*365.25)
            if(time.ge.start(i).and.start(i).lt.begin)
     1              x(k,m)=v(i,m)*(time-begin)/(sample*365.25)
          enddo
        enddo
c
c white noise: for this and other noises, simply compute each component
c  separately
c
        do m=1,3
          if(wh(i,m).ne.0) call gwhite(aux1,ne)
          do k=1,ne
            x(k,m)=x(k,m)+aux1(k)*wh(i,m)
          enddo
        enddo
c
c flicker noise
c
        do m=1,3
          if(fl(i,m).ne.0) call flicker(aux1,ne)
          do k=1,ne
            x(k,m)=x(k,m)+(aux1(k)-aux1(in1))*fl(i,m)
          enddo
        enddo
c
c random-walk noise
c
        do m=1,3
          if(rw(i,m).ne.0) call redwlk(aux1,ne)
          do k=1,ne
            x(k,m)=x(k,m)+(aux1(k)-aux1(in1))*rw(i,m)
          enddo
        enddo
c
c power-law noise
c
c       do m=1,3
c         call plnoise(pows(i,m),powe(i,m),aux1,ne)
c         do k=1,ne
c           x(k,m)=x(k,m)+aux1(k)-aux1(in1)
c         enddo
c       enddo
c
c fault slip--in this case, find all three series at once, since this
c  is what the dislocation routine computes
c
        call slipsig(slat(i),slong(i),aux1,aux2,aux3,ne)
        do k=1,ne
          x(k,1)=x(k,1)+aux1(k)-aux1(in1)
          x(k,2)=x(k,2)+aux2(k)-aux2(in1)
          x(k,3)=x(k,3)+aux3(k)-aux3(in1)
        enddo
c
c common-mode noise
c
        call cmode(slat(i),slong(i),aux1,aux2,aux3,ne)
        do k=1,ne
          x(k,1)=x(k,1)+aux1(k)-aux1(in1)
          x(k,2)=x(k,2)+aux2(k)-aux2(in1)
          x(k,3)=x(k,3)+aux3(k)-aux3(in1)
        enddo
c
c annual cycles
c
        do m=1,3
          call discos(aux1,ne,365,as(i,m),ps(i,m),aux2,aux3)
          do k=1,ne
            x(k,m)=x(k,m)+aux1(k)*annual(i,m)
          enddo
        enddo
c
c write out series, skipping gaps
c
        do k=1,ne
          time=begin + (k-1)*sample
          iok=1
          if(time.lt.start(i).or.time.gt.stop(i)) iok=0
          if(iok.eq.1.and.ngap(i).gt.0) then
            do n=1,ngap(i)
              if(gaps(i,n,1).lt.time.and.gaps(i,n,2).gt.time) iok=0
            enddo
          endif
          if(iok.eq.1) then
             call jul2dat(time,itm)
             tmjd=time-2400000.5
             call toyd(itm,it)
             write(luo,110) (itm(l),l=1,3),(x(k,n),n=1,3),tmjd
 110         format(i4,'-',i2.2,'-',i2.2,3(',',f10.3),',',f11.4)
C             write(luo,110) (it(l),l=1,2),(x(k,n),n=1,3)
C 110         format(i4,'.',i3.3,3f10.3)
          endif
        enddo
        close(luo)
 13     continue
      enddo
      name(1:15)=dnam
      name(16:19)='.fin'
      name(20:24)='    '
      open(luo,file=name(1:19),status='unknown')
      write(luo,114),dnam,dnam,dnam,dnam,dnam
 114  format('tar -cf ',a15,'.tar *.',a15,'.csv'
     1                     ,/,'rm *.',a15,'.csv',/,
     2       'gzip ',a15,'.tar',/,
     3       'rm ',a15,'.fin')
      close(luo)
      write(name(1:22),120) dnam
 120  format('sh ',a15,'.fin')
      call system(name(1:22))
      return
      end
c=======================================================================
      subroutine slipsig(rlat,rlong,x1,x2,x3,n)
c
c  Returns a time series of signal produced at lat rlat, longitude rlong,
c   by slip on fault planes.
c   Array x, of length n, contains the results.
c
      parameter (maxpl=5000)
      parameter (maxsl=500000)
      save ro
      save do0,to0,so0,do9,to9,so9,don,ton,son,wr
      character*1 lcode,dcode,locat
      double precision time
      double precision begin,end,sample
      double precision to
      dimension x1(*),x2(*),x3(*)
      dimension d(3),t(3),s(3),wr(3)
      dimension do0(3),to0(3),so0(3)
      dimension do9(3),to9(3),so9(3)
      dimension don(3),ton(3),son(3)
c
c common block for overall parameters
c
      common/overtime/begin,end,sample,geog(4)
c
c common block for fault-plane geometry
c
      common/fltgeo/gt1(maxpl),gg1(maxpl),gt2(maxpl),gg2(maxpl),
     1 lcode(maxpl),dcode(maxpl),dep(maxpl),wid(maxpl),dip(maxpl),
     2 lst(maxpl),lse(maxpl),nflt
c
c common block for fault slips
c
      common/slips/time(maxsl),rake(maxsl),slip(maxsl),nslp
c
c common block for full parameter set for a slip event
c
      common/eqparm/slat,slong,depth,stk,dp,rak,al,aw,sl,locat
c
      data dr/.0174533/,ro/-12345./
      to=begin
c
c go through slip events by fault; if they fall within a time step, get
c  the fault geometry and compute the effects
c
c the fault geometry is converted from two end coordinates to one
c center coordinate, strike, and length
c
      do i=1,n
        x1(i)=0
        x2(i)=0
        x3(i)=0
      enddo
      do k=1,nflt
c
c lst(k)=0 if there are no slips for this segment
c
        if(lst(k).gt.0) then
          call topo(gt1(k),gg1(k),gt2(k),gg2(k),al,stk)
          call topod(gt1(k),gg1(k),al/2,stk,slat,slong)
          depth= dep(k)
          dp   = dip(k)
          aw   = wid(k)
          locat= lcode(k)
          sl  = 1
          rak = 0
          call stpsub(rlat,rlong,so0,to0,do0)
          rak = 90
          call stpsub(rlat,rlong,so9,to9,do9)
          rak = 370
          call stpsub(rlat,rlong,son,ton,don)
c
c start of loop over slips associated with each fault; time must fall
c  within range of interest
c
          do j=lst(k),lse(k)
            if(time(j).ge.begin.and.time(j).le.end) then
              i=nint((time(j)-to)/sample) + 1
c
c  i is the location in the time series
c
c  if rake changes, recompute associated weights
c
              rak  = rake(j)
              if(rak.ne.ro) then
                if(rak.lt.370) then
                   wr(1) = cos(dr*rak)
                   wr(2) = sin(dr*rak)
                   wr(3) = 0.
                endif
                if(rak.ge.370) then
                   wr(1) = 0.
                   wr(2) = 0.
                   wr(3) = 1.
                endif
                ro=rak
              endif
c
c now find the displacement, strain, and tilt
c
              sl   = slip(j)
              do m=1,3
                d(m) = sl*(wr(1)*do0(m) + wr(2)*do9(m) + wr(3)*don(m))
                t(m) = sl*(wr(1)*to0(m) + wr(2)*to9(m) + wr(3)*ton(m))
                s(m) = sl*(wr(1)*so0(m) + wr(2)*so9(m) + wr(3)*son(m))
              enddo
c
c d is south, east, up, convert to e n u
c
              x1(i)=x1(i)+d(2)
              x2(i)=x2(i)-d(1)
              x3(i)=x3(i)+d(3)
            endif
          enddo
        endif
      enddo
c
c now sum the series to get the cumulative displacement
c
      do i=2,n
        x1(i)=x1(i-1)+x1(i)
        x2(i)=x2(i-1)+x2(i)
        x3(i)=x3(i-1)+x3(i)
      enddo
      return
      end
c=======================================================================
      subroutine stpsub(rlat,rlong,e2,t2,d2)
c   for a given dislocation source with parameters specified in
c  common block eqparm, finds the strains, tilts, and  displacements at
c  a location with latitude rlat, longitude rlong (the half-space formulae
c  are used, so the source should not be too far from the receiver)
c
c   $$$calls topo,fdefor,stran,van
c
      save e1,t1,d1,e0,d0,t0,dr
      dimension e2(3),t2(3),d2(3)
      dimension e1(3),t1(3),d1(3)
      dimension e0(3),t0(3),d0(3)
c
c   source parameters are:
c     slat, slong - N lat and E long of source (degrees); if locat is
c                   c, these are the coordinates of the center of the plane; if
c                   t, the coordinates of the center of the top edge; if
c                   o, the coordinates of the center of the surface trace
c                   (projected up-dip if need be)
c     depth       - depth of source (km) (mid-depth of plane)
c     str,dip,rake- strike and dip of fault, rake of motion
c                   (rake is -180 for rl strike-slip)
c     al, aw      - length (horizontal) and width (down-dip) of fault (km)
c     slip        - fault slip (m)
c
c
      character*1 locat
      common/eqparm/slat,slong,depth,str,dip,rake,al,aw,slip,locat
      common/elascn/pr
      data dr/.0174533/
      pr=0.25
      do  i=1,3
        e2(i) = 0.
        d2(i) = 0.
        t2(i) = 0.
      enddo
      call topo(slat,slong,rlat,rlong,del,az)
c  compute coordinates in fault-centered system
      theta = str - az
      xl = del*cos(dr*theta)
      yl = del*sin(dr*theta)
c  if necessary, adjust for the means of specifying the fault location
c  (the depth already being set to the mid-depth)
      if((abs(abs(dip)-90.).gt.0.05).and.(locat.eq.'o'))
     1                 yl = yl + depth/tan(dr*dip)
      if(locat.eq.'t') yl = yl + .5*aw*cos(dr*dip)
c  scale slip to be in km. rake of 370 is fault opening
      if(rake.ne.370.) then
       uh = slip*cos(dr*rake)/1000.
       uv = slip*sin(dr*rake)/1000.
       un = 0.
      elseif(rake.eq.370.) then
       uh = 0.
       uv = 0.
       un = slip/1000.
      endif
      call fdefor(depth,al,aw,dip,uh,uv,un,xl,yl,d0,t0,e0)
c  rotate motions into geographical coordinates (1 south, 2 east)
      rot = str + 180.
      call van(d0,rot,d1)
      d1(3) = d0(3)
      call van(t0,rot,t1)
      t1(3) = t0(3)
      call stran(e0,rot,e1)
c
c   scale to mm, nrad, nanostrain, and accumulate
c
      do i=1,3
        e2(i) = 1.e9*e1(i)
        d2(i) = 1.e6*d1(i)
        t2(i) = 1.e9*t1(i)
      enddo
      return
      end
c=======================================================================
      subroutine fdefor(depth,al,aw,dip,uh,uv,un,xr,yr,dis,tlt,str)         
c--------------------------------------
c     subroutine compute deformation for a finite fault,
c     by manipulating calling variables into
c     Chinnery's (double bar) notation ||.  See Okada
c     page 1143.  This routine offsets the variables 
c     across the finite fault surface and then calls
c     the calculating routine (static.f) a total of 4 times.
c     calling parameters:
c        depth:     depth of center of fault surface
c           al:     fault length
c           aw:     fault width
c          dip:     fault dip angle measured at depth reletive
c                   to horizontal
c           uh:     horizontal slip of fault
c           uv:     vertical slip of fault
c           un:     normal slip of fault
c           xr:     receiver location along fault, relative to
c                   center point
c           yr:     receiver location perpendicular
c          dis:     3 element array where dislocations will be
c                   returned. dis(1) is x displacement, dis(2)
c                   is y and dis(3) is z with z positive up.
c          str:     3 element array where strain components are
c                   returned. str(1) is dux/dx, str(2) is
c                   duy/dy, str(3) is (duy/dx + dux/dy)/2
c          tlt:     3 element array where tile components are
c                   returned. tlt(1) is -duz/dx and tlt(2) is -duz/dy
c                   (to match tilt convention that + value is down
c                   in the direction indicated) tlt(3) is the rotation
c                   around a horizontal axis
c
c     All calling parameters are single precision reals.
c
c     program by: Y. Okada     February 1985
c     reworked by: H. Johnson  July 1988
c                  D  Agnew    August 2005 (tlt(3) added)
c--------------------------------------
      implicit double precision (a-h,o-z)
      real depth, dip, al, aw, pr, xr, yr, uh, uv, un, dis, str, tlt
      integer j, k, sign
      dimension dis(*), str(*), tlt(*), tmp1(3), tmp2(4),tmp3(2),
     1 dtmp(3),stmp(4),ttmp(2)
c  common block elascn holds the poisson's ratio, which may be set externally,
c  though not with a data statement
      common/elascn/pr
      data d0, d1, f0, f1, eps/0.0, 1.0, 0.d0, 1.d0, 1.d-3/
      rad=6.283185307179586d0/360.d0
c   compute  alp, a material property parameter made up of
c   Lame's constants lambda and mu: alp = mu / (lambda + mu)
      alp = 1. - 2*pr
      sd=dsin(dip*rad)
      cd=dcos(dip*rad)
      if(dabs(cd) .le. eps) then
        cd=f0
        if(sd .ge. f0) then
          sd=f1
        else sd=-f1
        endif
      endif
c  reset depth and coordinates to make origin the corner of the fault,
c  as used by Okada
      dep = depth + (aw/2.)*sd
      x   = xr    + al/2.
      y   = yr    + (aw/2.)*cd
c  proceed with the Okada computation
      p=y*cd + dep*sd
      q=y*sd - dep*cd
      do 1 i=1,2
        dis(i)=d0
        dis(i+1)=d0
        str(i)=d0
        str(i+1)=d0
        tlt(i)=d0
        tlt(i+1)=d0
        dtmp(i)=f0
        dtmp(i+1)=f0
        stmp(i)=f0
        stmp(i+2)=f0
        ttmp(i)=f0
        tmp1(i)=f0
        tmp1(i+1)=f0
        tmp2(i)=f0
        tmp2(i+2)=f0
        tmp3(i)=f0
1     continue
      do 2 k=1,2
        if(k .eq. 1) then
          eta=p
        else
          eta=p-aw
        endif
        do 3 j=1,2
          if(j .eq. 1) then
            xi=dble(x)
          else
            xi=dble(x)-dble(al)
          endif
          if(j+k .eq. 3) then
            sign=-d1
          else
            sign=d1
          endif
          call static(alp,xi,eta,q,sd,cd,dble(uh),
     :                dble(uv),dble(un),tmp1,tmp2,tmp3)
          do 4 i=1,2
            dtmp(i) = dtmp(i) + sign*tmp1(i)
            stmp(i) = stmp(i) + sign*tmp2(i)
            ttmp(i) = ttmp(i) + sign*tmp3(i)
4         continue
          dtmp(3) = dtmp(3) + sign*tmp1(3)
          stmp(3) = stmp(3) + sign*tmp2(3)
          stmp(4) = stmp(4) + sign*tmp2(4)
3       continue
2     continue
c   find tensor strain, and change sign of tilt
      str(1)=stmp(1)
      str(2)=stmp(4)
      str(3)=(stmp(2)+stmp(3))/2
      do 5 i=1,2
         dis(i) = dtmp(i)
         dis(i+1) = dtmp(i+1)
         tlt(i) = -ttmp(i)
 5    continue
      tlt(3)=(stmp(2)-stmp(3))/2
      return
      end
      subroutine static(alp,xi,eta,q,sd,cd,uh,uv,un,dis,str,tlt)
c--------------------------------------
c     Subroutine to actually compute the dislocation
c     components outlined by equations in Okada on pages
c     1144 to 1148.  Calling parameters are the same
c     as for routine sfault.f except:
c
c          xi:     The offset point on the fault surface
c                  in the length direction.
c                  This takes on values at the four corners
c                  of the finite size fault surface.
c         eta:     Same as above but in width direction.
c
c     program by: Y. Okada     February 1985
c     reworked by: H. Johnson  July 1988
c--------------------------------------
      implicit double precision (a-h,o-z)
      dimension dis(3), str(4), tlt(2)
      data f0,f1,f2/ 0.d0, 1.d0, 2.d0/
      pi2=6.283185307179586d0
      xi2=xi*xi
      eta2=eta*eta
      q2=q*q
      r2=xi2+eta2+q2
      r=dsqrt(r2)
      r3=r*r2
      d=eta*sd-q*cd
      y=eta*cd+q*sd
      reta=r+eta
      if(reta .lt. f0) reta=f0
      rd=r+d
      rrd=f1/(r*rd)
      if(q .eq. f0) then
        tt=f0
      else
        tt=datan(xi*eta/(q*r))
      endif
      if(reta .eq. f0) then
        re=f0
        dle=-dlog(r-eta)
      else
        re=f1/reta
        dle=dlog(reta)
      endif
      rrx=f1/(r*(r+xi))
      rre=re/r
      axi=(f2*r+xi)*rrx*rrx/r
      aeta=(f2*r+eta)*rre*rre/r
      if(cd .ne. f0) then
        td=sd/cd
        x=dsqrt(xi2+q2)
        if(xi .eq. f0) then
          a5=f0
        else
          a5=alp*f2/cd*datan((eta*(x+q*cd)+x*(r+x)*sd)/(xi*(r+x)*cd))
        endif
        a4=alp/cd*(dlog(rd)-sd*dle)
        a3=alp*(y/rd/cd-dle)+td*a4
        a1=-alp/cd*xi/rd-td*a5
        c1=alp/cd*xi*(rrd-sd*rre)
        c3=alp/cd*(q*rre-y*rrd)
        b1=alp/cd*(xi2*rrd-f1)/rd-td*c3
        b2=alp/cd*xi*y*rrd/rd-td*c1
      else
        rd2=rd*rd
        a1=-alp/f2*xi*q/rd2
        a3=alp/f2*(eta/rd+y*q/rd2-dle)
        a4=-alp*q/rd
        a5=-alp*xi*sd/rd
        b1=alp/f2*q/rd2*(f2*xi2*rrd-f1)
        b2=alp/f2*xi*sd/rd2*(f2*q2*rrd-f1)
        c1=alp*xi*q*rrd/rd
        c3=alp*sd/rd*(xi2*rrd-f1)
      endif
      a2=-alp*dle-a3
      b3=-alp*xi*rre-b2
      b4=-alp*(cd/r+q*sd*rre)-b1
      c2=alp*(-sd/r+q*cd*rre)-c3
      do 1 i=1,2
        dis(i)=f0
        dis(i+1)=f0
        str(i)=f0
        str(i+2)=f0
        tlt(i)=f0
1     continue
      if(uh .ne. f0) then
        u1=uh/pi2
        req=rre*q
        dis(1)=dis(1)-u1*(req*xi+tt+a1*sd)
        dis(2)=dis(2)-u1*(req*y+q*cd*re+a2*sd)
        dis(3)=dis(3)-u1*(req*d+q*sd*re+a4*sd)
        str(1)=str(1)+u1*(xi2*q*aeta-b1*sd)
        str(2)=str(2)+u1*(xi2*xi*(d/(eta2+q2)/r3-aeta*sd)-b2*sd)
        str(3)=str(3)+u1*(xi*q/r3*cd+(xi*q2*aeta-b2)*sd)
        str(4)=str(4)+u1*(y*q/r3*cd+(q*sd*(q2*aeta-f2*rre)
     :      -(xi2+eta2)/r3*cd-b4)*sd)
        tlt(1)=tlt(1)+u1*(-xi*q2*aeta*cd+(xi*q/r3-c1)*sd)
        tlt(2)=tlt(2)+u1*(d*q/r3*cd+(xi2*q*aeta*cd-sd/r+y*q/r3-c2)*sd)
      endif
      if(uv .ne. f0) then
        u1=uv/pi2
        sdcd=sd*cd
        dis(1)=dis(1)-u1*(q/r-a3*sdcd)
        dis(2)=dis(2)-u1*(y*q*rrx+cd*tt-a1*sdcd)
        dis(3)=dis(3)-u1*(d*q*rrx+sd*tt-a5*sdcd)
        str(1)=str(1)+u1*(xi*q/r3+b3*sdcd)
        str(2)=str(2)+u1*(y*q/r3-sd/r+b1*sdcd)
        str(3)=str(3)+u1*(y*q/r3+q*cd*rre+b1*sdcd)
        str(4)=str(4)+u1*(y*y*q*axi-(f2*y*rrx+xi*cd*rre)*sd+b2*sdcd)
        tlt(1)=tlt(1)+u1*(d*q/r3+q*sd*rre+c3*sdcd)
        tlt(2)=tlt(2)+u1*(y*d*q*axi-(f2*d*rrx+xi*sd*rre)*sd+c1*sdcd)
      endif
      if(un .ne. f0) then
        u1=un/pi2
        sdsd=sd*sd
        dis(1)=dis(1)+u1*(q2*rre-a3*sdsd)
        dis(2)=dis(2)+u1*(-d*q*rrx-sd*(xi*q*rre-tt)-a1*sdsd)
        dis(3)=dis(3)+u1*(y*q*rrx+cd*(xi*q*rre-tt)-a5*sdsd)
        str(1)=str(1)-u1*(xi*q2*aeta+b3*sdsd)
        str(2)=str(2)-u1*(-d*q/r3-xi2*q*aeta*sd+b1*sdsd)
        str(3)=str(3)-u1*(q2*(cd/r3+q*aeta*sd)+b1*sdsd)
        str(4)=str(4)-u1*((y*cd-d*sd)*q2*axi-f2*q*sd*cd*rrx
     :      -(xi*q2*aeta-b2)*sdsd)
        tlt(1)=tlt(1)-u1*(q2*(sd/r3-q*aeta*cd)+c3*sdsd)
        tlt(2)=tlt(2)-u1*((y*sd+d*cd)*q2*axi+xi*q2*aeta*sd*cd
     :      -(f2*q*rrx-c1)*sdsd)
      endif
      return
      end
c=======================================================================
      subroutine stran(ein,ang,eout)
c  for plane strain ein (expressed as a vector e11, e22, e12),
c  computes the strain (expressed similarly) for a coordinate
c  system rotated counterclockwise by an angle ang (in degrees),
c  the result being returned in eout
c    $$$$$calls only system routines
      dimension ein(3),eout(3)
      data dr/.0174533/
      a = dr*ang
      c = cos(dr*ang)
      s = sin(dr*ang)
      eout(1) = ein(1)*c*c + ein(2)*s*s + 2.*ein(3)*c*s
      eout(2) = ein(1)*s*s + ein(2)*c*c - 2.*ein(3)*c*s
      eout(3) = (-ein(1)+ein(2))*c*s + ein(3)*(c**2 - s**2)
      return
      end
c=======================================================================
      subroutine van(vin,ang,vout)
c  for a plane vector vin computes the vector for a coordinate
c  system rotated counterclockwise by an angle ang (in degrees),
c  the result being returned in vout
c    $$$$$calls only system routines
      dimension vin(2),vout(2)
      data dr/.0174533/
      a = dr*ang
      c = cos(dr*ang)
      s = sin(dr*ang)
      vout(1) = vin(1)*c + vin(2)*s
      vout(2) = -vin(1)*s + vin(2)*c
      return
      end
c=======================================================================
      subroutine cmode(rlat,rlong,x1,x2,x3,n)
c
c  finds the contribution from the common-mode series at a given location,
c   and computes the (three) resulting series
c
      parameter (maxep=73100)
      parameter (maxcom=10)
      dimension x1(*),x2(*),x3(*)
      dimension wt(maxcom)
c
c common block for common-mode noise parameters
c
      common /cmnoi/clat(maxcom),clong(maxcom),
     1  cwh(maxcom,3),cfl(maxcom,3),crw(maxcom,3),
     2  cpows(maxcom,3),cpowe(maxcom,3),ncomm
c
c common block for common-mode noise series
c
      common /cmser/cmseries(maxep,maxcom,3)
c
c if no common-mode series, exit
c
      do i=1,n
        x1(i)=0.
        x2(i)=0.
        x3(i)=0.
      enddo
      if(ncomm.eq.0) return
c
c find weights, using inverse distance
c
      wtn=0
      do i=1,ncomm
        call topo(rlat,rlong,clat(i),clong(i),wt(i),tmp)
        if(wt(i).lt.1) wt(i)=1.
        wt(i)=1/wt(i)
        wtn=wtn+wt(i)
      enddo
      do i=1,ncomm
        wt(i)=wt(i)/wtn
      enddo
      do i=1,n
        do k=1,ncomm
          x1(i)=x1(i)+wt(k)*cmseries(i,k,1)
          x2(i)=x2(i)+wt(k)*cmseries(i,k,2)
          x3(i)=x3(i)+wt(k)*cmseries(i,k,3)
        enddo
      enddo
      return
      end
c=======================================================================
c ***********************************************************************
c *                                                                     *
c *   Routines below for reading and processing script commands are     *
c *     provided courtesy of R. L. Parker; specific commands in review  *
c *     listing in scanrp, and checking in icheck, have been modified   *
c *     for this code.                                                  *
c *                                                                     *
c ***********************************************************************
c=======================================================================
      subroutine scanrp
c
c$$$$ calls clear getchr icheck
c
c  Input routine for commands 
c
c  Reads from the standard input until eof or code "execute " or "quit".
c  Saves lines in the input store  /store/ for later retrieval by
c  getarr, getone or getchr.
c
c  Prints a glossary upon request
c
      parameter (inmx=500)
      character*4 code
      character*80 input(inmx),line
      common /store/ input
      common /ndict/ iecho,nin
c
      if (nin .eq. 0) write(6,'(a)') ' ',
     $'Enter commands for producing fakenet file (? for help)'
c
      do 1500 l=nin+1, inmx
        read(*,'(80a)', end=3000) line
        if (line(1:4).eq.'cont' .or. line(1:4).eq.'exec') goto 2000
c  List a glossary of codes
        if (line(1:1) .eq. '?') then
c     $'output filename:          Enter file for output file (optional)',
          write(6, '(a/a/(2x,a))')
     $'Enter commands from the following list:',
     $'?:        Remind me of the command list again',
     $'execute:           Quit reading commands and begin calculations',
     $'location filename:        Enter file of station list (required)',
     $'begin yr day [h m s]:     Set start time to be taken (required)',
     $'end yr day [h m s]:         Set end time to be taken (required)',
     $'sample days:             Set sample interval in days (required)',
     $'area south north west east:     Set region to select (optional)',
     $'faults filename:     Enter file for faults and slips (optional)',
     $'velocity filename:          Enter file of velocities (optional)',
     $'performance filename:         Enter file of up times (optional)',
     $'seed iseed:     Set seed for random number generator (optional)',
     $'mseed mseed:    Set seed for secnd random number gen (optional)',
     $'verror vn ve vu:     Set perturbations to velocities (optional)',
     $'stanoise filename:       Enter file of station noise (optional)',
     $'samnoise filename:          Enter file of noise menu (optional)',
     $'annual filename:     Enter file of annual-cycle amps (optional)',
     $'rannual filename:    Enter file of annual-cycle menu (optional)',
     $'modulate amp phase:        Set modulations of annual (optional)',
     $'common filename:    Enter file for common-mode noise (optional)',
     $'logout filename:              Enter name of log file (optional)'
          write(6, '(2x,a)')
     $'review:                           Display current command stack',
     $'clear command:     Delete most recent occurrence of the command',
     $' '
c
c  Review the command stack
        elseif (line(1:4) .eq. 'revi') then
          write(6,'(5x,a)')' ', '=================== ',
     $    (input(i)(1:60),i=1,nin),'=================== '
c
        elseif (line(1:1) .ne. ' ') then
          if (line(1:4) .eq. 'echo') iecho=-iecho
          if (line(1:4) .eq. 'quit') then
            write(6,'(/a)') 'Normal termination'
            stop
          endif
          nin=nin + 1
          if (nin .gt. inmx) then
            write(6,'(a)') '>>>> Too many commands - memory full'
            stop
          endif
          call icheck(line)
          input(nin)=line
c  Clear a command and clear the clear command
          if (line(1:4) .eq. 'clea') then
            call getchr('clear', code, klear)
            call clear (code)
            nin=nin - 1
          endif
        endif
 1500 continue
c
 2000 continue
      n1=max(1, nin-24)
      write(6,'(5x,a)')' ', '=================== ',
     $(input(i)(1:60),i=n1,nin),'=================== '
      return
c
 3000 stop
      end
c=======================================================================
      subroutine icheck(line)
c
c$$$$ calls no other routines
c
c  Checks the command fragment com against a list; appends a
c  warning if com is not present in the list.
      character*80 line
      character*4 com
c
c   lines below are to handle changes in commands made in later versions,
c  to provide backwards compatibility
c
      if(line(1:4).eq.'logo') line(1:4)='logf'
      com=line(1:4)
      if (0.eq.
     1 index('area begi end  loca velo outp perf samp faul',com)+
     2 index('seed verr logf stan samn comm annu modu rann',com)+
     3 index('msee',com))
     4 line=line(1:20)//' <<<<<<< Unrecognized command'
      return
      end
