c
c  $Id: fakesubs.f,v 1.4 2012/11/25 06:06:00 agnew Exp agnew $
c
c  $Log: fakesubs.f,v $
c  Revision 1.4  2012/11/25 06:06:00  agnew
c  added second random-number and gaussian programs
c
c  Revision 1.3  2010/03/05 00:45:36  agnew
c  Moved sinus from fakenet
c
c  Revision 1.2  2010/03/05 00:40:28  agnew
c  Added discos, bbrid; moved gwhite, redwlk, and flicker from fakenet
c
c  Revision 1.1  2009/06/02 22:20:37  agnew
c  Initial revision
c
c
c=======================================================================
      subroutine discos(x,n,np,ascl,pscl,a,p)
c
c  creates, in x, a sinusoidal series of length n and period np. The amplitude
c   is modulated by a Brownian bridge series of length n, and the phase is
c   modulated by a periodic Brownian bridge series of period np (this
c   series goes to zero every np terms)
c
c  the initial phase is set to a uniformly-distributed random value between
c   zero and 2 pi
c
c  ascl and pscl set the scaling for the two Brownian bridge series. The
c   arrays  a  and  p  are storage for these series, and should be externally  
c   dimensioned at least of length n
c
      dimension x(*),a(*),p(*)
c
c   make amplitude series
c
      call bbrid(a,n)
      do i=1,n
        a(i)=ascl*a(i)
      enddo
c
c   make phase series, which has m periods of length np; the remainder is
c    also made into a bridge
c
      m=n/np
      mr=n-m*np
      l=1
      do i=1,m
        call bbrid(p(l),np)
        l=l+np
      enddo
      call bbrid(p(l),mr)
c
c   scale phase to radians and make x
c
c     phs=0.0
      phs=2*3.14159*genrand_real3()
      do i=1,n
        ph=pscl*p(i)*(2*3.13159)/180 + phs
        x(i)=(1+a(i))*cos((2*3.14159*i)/np + ph)
      enddo
      return
      end
c=======================================================================
      subroutine bbrid(x,n)
c
c    returns, in x a Brownian bridge of length n, formed by taking a random
c     walk and subtracting the first value and a slope to constrain the
c     series to be zero at both ends
c
      dimension x(*)
      call redwlk(x,n)
      sl=(x(n)-x(1))/(n-1)
      st=x(1)
      do i=1,n
        x(i) = x(i)-st-sl*(i-1)
      enddo
      return
      end
c=======================================================================
      subroutine gwhite(x,n)
c
c    computes n values of a Gaussian distribution of white noise in array x
c
      double precision genrand_real3
      dimension x(*),v(2)
      do i=1,n,2
  3      continue
         v(1)=genrand_real3()
         v(2)=genrand_real3()
         r=0
         do j=1,2
           v(j)=2*v(j)-1
           r=r+v(j)**2
         enddo
         if(r.ge.1) go to 3
         xx = sqrt(-2.*alog(r)/r)
         x(i) = v(1)*xx
         if(i+1.le.n) x(i+1) = v(2)*xx
      enddo
      return
      end
c=======================================================================
      subroutine redwlk(x,n)
c
c    computes n values of a random walk in x
c
c  scale by 3.16 to have the (extrapolated) Nyquist intercept be 0 db
c
      dimension x(*)
      call gwhite(x,n)
      do i=2,n
        x(i)=x(i-1)+x(i)
      enddo
      return
      end
c=======================================================================
      subroutine flicker(x,n)
c
c computes n values of flicker noise in array x
c
c  scale by 3.35 to have the Nyquist intercept be 0 db
c
      double precision genrand_real3
      dimension x(*),xp(32)
      nt=1
      do i=1,32
        xp(i)=genrand_real3()
      enddo
      do j=1,n
        x(j) = 0.  
        do l = 1,32
          x(j) = x(j) + xp(l)      
        enddo
        x(j) = x(j)-16.
        l = 2      
        nt = nt + 1
        nn = nt     
        xp(1)=genrand_real3()
 3      m = nn/2    
        if(2*m.eq.nn) then
           xx=genrand_real3()
           xp(l) = xx
           l = l + 1  
           nn = m      
           go to 3   
        endif
      enddo
      return
      end
c=======================================================================
      subroutine sinus(x,c,s,p,ne)
c
c constructs a sinusoidal series, given the sine and cosine parts, and
c  the period (in samples). The cosine and sine parts are assumed to be
c  such that the first term will have the value of the cosine part.
c
      dimension x(*)
      double precision pi2,om,p
      data pi2/6.283185307179586d0/
      om=pi2/p
      do i=1,ne
        a=om*(i-1)
        x(i)=c*cos(a)+s*sin(a)
      enddo
      return
      end
c=======================================================================
      subroutine centroid(rlat,rlong,np,clat,clong)
c
c  find the centroid of np points, as the average of the geocentric
c  Cartesian coordinates
c
      parameter (maxpl=1000)
      double precision txyz,cxyz
      dimension rlat(*),rlong(*)
      dimension txyz(3),cxyz(3)
      do j=1,3
        cxyz(j)=0.d0
      enddo
      do i=1,np
        call toxyz(rlat(i),rlong(i),0.,txyz)
        do j=1,3
          cxyz(j)=txyz(j)+cxyz(j)
        enddo
      enddo
      do j=1,3
        cxyz(j)=cxyz(j)/np
      enddo
      call frmxyz(cxyz,clat,clong,ch)
      return
      end
c=======================================================================
      subroutine toxyz(rlat,rlong,rht,cart)
c
c     Convert geodetic coordinates to Cartesian coordinates
c
c   Based on geoxyz routine in GAMIT
c
      double precision cart,f,dr,e2,curvn,semi
      dimension cart(*)
      data semi/6378137.d0/,finv/298.257223563/,pi/3.1415926536/
      dr = pi/180.d0
      f  = 1./finv
      e2 = 2.*f - f*f
      sinlat = sin(dr*rlat)
      coslat = cos(dr*rlat)
      sinlon = sin(dr*rlong)
      coslon = cos(dr*rlong)
      curvn  = semi/(sqrt(1.-e2*sinlat*sinlat))
      cart(1) = (curvn+rht)*coslat*coslon
      cart(2) = (curvn+rht)*coslat*sinlon
      cart(3) = (curvn*(1.-e2)+rht)*sinlat
      return
      end
c=======================================================================
      subroutine frmxyz(cart,rlat,rlong,rht)
c
c     Convert Cartesian coordinates to geodetic coordinates
c
c   Based on geoxyz routine in GAMIT
c
      double precision cart,f,dr,e2,curvn,semi
      dimension cart(*)
      data semi/6378137.d0/,finv/298.257223563/,pi/3.1415926536/
      dr = pi/180.
      f  = 1./finv
      e2 = 2.*f - f*f
      rlong = atan2(cart(2),cart(1))/dr
c starting value for latitude iteration
      sq = sqrt(cart(1)**2+cart(2)**2)
      rlat0 = atan2(cart(3)/sq,1.-e2)
      rlat = rlat0
  3   sinlat = sin(rlat)
      curvn= semi/(sqrt(1.-e2*sinlat*sinlat))
      rlat= atan2((cart(3)+e2*curvn*sinlat),sq)
      if(abs(rlat-rlat0).lt.1.e-8) go to 5
      rlat0 = rlat
      go to 3
  5   cutoff= 80.*pi/180.
      if(rlat.le.cutoff) then
        rht = (sq/cos(rlat))-curvn
      else
        rht = cart(3)/sin(rlat)-curvn+e2*curvn
      endif
      rlat = rlat/dr
      return
      end
c=======================================================================
      subroutine topo(slat,slong,rlat,rlong,del,az)
c  for small distances, computes the distance from point s to point r
c  del is the distance in km, az the azimuth (cw from n)
c   uses "topographic" formula (bomford, 4th ed, p. 123)
      data dr/.017453293/,flt/3.35289e-3/,eqr/6378.15/
      if(rlat.eq.slat.and.rlong.eq.slong) then
        del=0.
        az=0.
        return
      endif
      e = sqrt(2*flt - flt*flt)
      phm = .5*dr*(slat+rlat)
      sphm = sin(phm)
      rho = eqr*(1-e*e)*(1 + 1.5*(e*sphm)**2)
      rnu = eqr*(1 + .5*(e*sphm)**2)
      da = dr*(rlong-slong)*sphm
      az = atan2(rnu*dr*(rlong-slong)*cos(phm),rho*dr*(rlat-slat))
     1     - .5*da
      caz = cos(az+.5*da)
      saz = sin(az+.5*da)
      if(abs(caz).gt.abs(saz)) del = dr*rho*(rlat-slat)/caz
      if(abs(caz).le.abs(saz)) del = dr*rnu*cos(phm)*(rlong-slong)/saz
      az = az/dr
      return
      end
c=======================================================================
      subroutine topod(slat,slong,del,az,rlat,rlong)
c  for small distances, computes the coordinates of point r relative
c  to point s, for del the distance in km, az the azimuth (cw from n)
c   uses "topographic" formula (bomford, 4th ed, p. 119)
      data dr/.017453293/,flt/3.35289e-3/,eqr/6378.15/
      e = sqrt(2*flt - flt*flt)
      phm = dr*slat
      sphm = sin(phm)
      rho = eqr*(1-e*e)*(1 + 1.5*(e*sphm)**2)
      rnu = eqr*(1 + .5*(e*sphm)**2)
      azt = dr*az
c  need preliminary estimate of new longitude and latitude
      rlong = slong + (del/rnu)*sin(azt)/(dr*cos(phm))
      rlat = slat + (del/rho)*cos(azt)/dr
c  now redo spheriod computations
      phm = .5*dr*(slat+rlat)
      sphm = sin(phm)
      rho = eqr*(1-e*e)*(1 + 1.5*(e*sphm)**2)
      rnu = eqr*(1 + .5*(e*sphm)**2)
      da = dr*(rlong-slong)*sphm
      rlat = slat + (del/rho)*cos(azt+.5*da)/dr
      rlong = slong + (del/rnu)*sin(azt+.5*da)/
     1                (dr*cos(phm+.5*dr*(slat-rlat)))
      return
      end
c=======================================================================
      double precision function dat2jul(it)
c
c$$$$ calls juldat
c
c Converts from yr-mon-day-hr-min-sec to Julian date
c
      double precision f
      dimension it(*)
      jd=juldat(it)
      f=it(4)/24.d0+it(5)/1440.d0+it(6)/86400.d0
      dat2jul=jd+f-0.5d0
      return
      end
c=======================================================================
      subroutine jul2dat(t,it)
c
c$$$$ calls datjul
c
c Converts from Julian date to yr-mon-day-hr-min-sec
c
      double precision t,tl,f,h
      dimension it(*)
      tl=t+0.5d0
      jd=int(tl)
      call datjul(jd,it)
      f=tl-jd
      h=24*f
      it(4)=int(h)
      f=h-it(4)
      h=60*f
      it(5)=int(h)
      f=h-it(5)
      h=60*f
      it(6)=nint(h)
      return
      end
c=======================================================================
      function juldat(it)
      dimension it(*)
c
c$$$$ calls no other routines
c
c  Finds Julian Date from Gregorian date, Algorithm from p. 604, Explanatory
c    Supplement Amer Ephemeris & Nautical Almanac (cf Comm CACM, 11, 657 (1968)
c    and 15, 918 (1972)) Valid for all positive values of Julian Date
c
      juldat=(1461*(it(1)+4800+(it(2)-14)/12))/4
     1     + (367*(it(2)-2-12*((it(2)-14)/12)))/12
     2     - (3*((it(1)+4900+(it(2)-14)/12)/100))/4+it(3)-32075
      return
      end
c=======================================================================
      subroutine datjul(jd,it)
c
c$$$$ calls no other routines
c
c  Converts Julian Date jd into year-month-day (Gregorian calendar) in
c    array it. Algorithm from p. 6??, Explanatory Supplement Amer Ephemeris
c    & Nautical Almanac
c
      dimension it(*)
      k = jd+68569
      n = 4*k/146097
      k = k - (146097*n+3)/4
      it(1) = 4000*(k+1)/1461001
      k = k - 1461*it(1)/4 + 31
      it(2) = 80*k/2447
      it(3) = k - 2447*it(2)/80
      k = it(2)/11
      it(2) = it(2) + 2 - 12*k
      it(1) = it(1) + 100*(n-49) + k
      return
      end
c=======================================================================
      subroutine toymd(it1,it2)
      dimension it1(*),it2(*)
c
c$$$$ calls no other routines
c
c  Converts times in it1 given as year and day to year-month-day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      mon(jj,m) = (12*(jj-29-m))/367 + 2 + (jj-200)/169
      it2(1)=it1(1)
      it2(2)=mon(it1(2),leap(it1(1)))
      it2(3) = it1(2) - idn(it2(2)) - leap(it2(1))*((9+it2(2))/12)
      it2(4) = it1(3)
      it2(5) = it1(4)
      it2(6) = it1(5)
      return
      end
c=======================================================================
      subroutine toyd(it1,it2)
      dimension it1(*),it2(*)
c
c$$$$ calls no other routines
c
c  converts times in it1 given as year-month-day to year and day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      it2(1)=it1(1)
      it2(2) = it1(3) + idn(it1(2)) + leap(it1(1))*((9+it1(2))/12)
      it2(3) = it1(4)
      it2(4) = it1(5)
      it2(5) = it1(6)
      return
      end
c=======================================================================
      function leap(iy)
c
c$$$$ calls no other routines
c
c returns 1 if year is a leap year, by Clavian (Gregorian) rule
c
      leap = 1 - (mod(iy,4)+3)/4
      if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) leap=0
      return
      end
c=======================================================================
c ***********************************************************************
c *                                                                     *
c *   Routines below for reading and processing script commands are     *
c *     provided courtesy of R. L. Parker; specific commands in review  *
c *     listing in scanrp, and checking in icheck, have been modified   *
c *     for this code.                                                  *
c *                                                                     *
c ***********************************************************************
c=======================================================================
      subroutine getarr(code, values, nwant, nfound)
c
c$$$$ calls ljust
c
c
c  Extracts an array of numbers from the input store.  That store is
c  a large array in common /store/ which has been filled earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned for numbers.
c  values  the real output array of values found.
c  nwant   the maximum number of numbers expected .
c  nfound  the number of numbers actually found in the input.
c          If the line contains fewer than nwant  values, this is the
c          value returned in  nfound.  If an error is discovered
c          nfound=-n, where  n  is the number of numbers properly
c          decoded.  If there are no numbers after the codeword
c          nfound=0.  Finally, if the code is absent from the store
c          nfound=-99 and the array is left undisturbed.
c
      parameter (inmx=500)
      character *80 input(inmx),line,local,char, code*4
      common /store/ input
      common /ndict/ iecho,nin
      dimension values(*)
c
c  Read the store in reverse order (Thus last entry is obeyed)
ce
      do 1010 lin=nin, 1, -1
        line=input(lin)
c  Check for code 
        if (code .eq. line(1:4)) goto 1020
 1010 continue
c  Code word not found
      nfound=-99
      return
c
 1020 if (iecho .ge. 1) write(6,'(2a)')'==> ',line
      n1=index(line, ' ')+1
      n2=index(line, '%')
      n2=80 + min(n2,1)*(n2 - 81)
      char=line(n1:n2)
c
      do 1500 n=1, nwant
        call ljust(char, local)
        lbl=index(local, ' ')
        if (lbl .eq. 1) goto 1510
        read (local, *, err=2000) values(n) 
        char=local(lbl:80)
 1500 continue
      n=nwant+1
 1510 nfound=n - 1
      return
c
 2000 print '(a)',' ',
     $'>>> Unreadable numbers in this input line:',line
      nfound=1 - n
      return
      end
c=======================================================================
      subroutine getone(code, value, nfound)
c
c$$$$ calls getarr
c
c  Extracts a single number from the input store.  That store is
c  a large array in common /store/ which has been filled earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned for numbers.
c  value   the real output variable containing the desired number.
c  nfound  is 1 if a number is successfully read in; it is zero
c          the number is absent or unreadable.  nfound = -99 if the
c          code is absent from the input store.
c
      character*4 code
      dimension v(1)
ce
      call getarr(code, v, 1, nfound)
      if (nfound .eq. 1) value=v(1)
      return
      end
c=======================================================================
      subroutine getint(code, number, nfound)
c
c$$$$ calls getarr
c
c  Extracts a single integer from the input store.  
c  See getone for details.
c
      character*4 code
      dimension v(1)
ce
      call getarr(code, v, 1, nfound)
      if (nfound .eq. 1) then
        number=nint(v(1))
        if (number .ne. v(1)) print '(4a,1p,g16.8/a,i10)',
     $  '>>> Warning: ',code,' expects an integer argument, but',
     $  ' found: ',v(1),'    Returns: ',number,' '
      endif
      return
      end
c=======================================================================
      subroutine getchr(code, char, nbytes)
c
c$$$$ calls ljust
c  Extracts a single character variable from the input store.  That 
c  store is a large array in common /store/ which has been filled 
c  earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned.
c  char    the character output variable containing the desired string.
c  nbytes  is the length of the string read; it is zero if
c          the line is blank after the code.  nbytes = -99 if the
c          code is absent from the input store.
c
      parameter (inmx=500)
      character *80 input(inmx),line, char*(*),  code*4
      common /store/ input
      common /ndict/ iecho,nin
c  Inspect the store in reverse order (thus reading latest entry)
ce
      do 1010 lin=nin, 1, -1
        line=input(lin)
c  Check for code word  
        if (code .eq. line(1:4)) goto 1020
 1010 continue
c  Code word not found
      nbytes=-99
      return
c
 1020 if (iecho .ge. 1) write(6,'(2a)')'==> ',line
c  
      n1=index(line, ' ')+1
      n2=index(line, '%')
      n2=80 + min(n2,1)*(n2 - 81)
c  Check for blank string
      nbytes=0
      if (line(n1:n2) .eq. ' ') return
c  Save in char everything from 1st non-blank to last non-blank
      call ljust(line(n1:n2), char)
      nn=min(n2-n1+1, len(char))
      do 1200 k=nn, 1, -1
        if (char(k:k) .ne. ' ') goto 1210
 1200 continue
 1210 nbytes= k
      return
      end
c=======================================================================
      subroutine clear(code)
c
c$$$$ calls no other routines
c
c  Removes the last command entry identified by code.
c  code    A 4-byte identifying code.  Only lines in the input store
c          beginning with this code are scanned.
c
      parameter (inmx=500)
      character *80 input(inmx),line, code*4
      common /store/ input
      common /ndict/ iecho,nin
c
c  Inspect the store in normal order
ce
      do 1010 lin= nin, 1, -1
        line=input(lin)
c  Check code  and clear the line if present, but only once
        if (code .eq. line(1:4)) then
          input(lin)=' --'
          return
        endif
 1010 continue
      return
      end
c=======================================================================
      subroutine ljust(str1, str2)
c
c$$$$ calls no other routines
c  Left justify: input character string str1 (up to 80 bytes in length)
c  is left justified, removing leading blanks, and returned in str2.
c  str1 and str2 may be set to the same variable in the call.
      character*(*) str1, str2, str3*80
c
ce
      str2=str1
      if (str1 .eq. ' ') return
c
      l1=len(str1)
      str3=str1
      do 1010 j=1, l1
        if (str1(j:j) .ne. ' ') goto 1011
 1010 continue
 1011 str2=str3(j:l1)
      return
      end
c=======================================================================
      blockdata iounit
      common /ndict/ iecho,nin
c  Don't echo command lines when read
      data iecho/-1/
c  Initial command line count
      data nin/0/
      end
c=======================================================================
c  A C-program for MT19937, with initialization improved 2002/1/26.
c  Coded by Takuji Nishimura and Makoto Matsumoto.
c
c  FORTRAN77 translation by Tsuyoshi TADA. (2005/12/19)
c
c  Before using, initialize the state by using init_genrand(seed)  
c  or init_by_array(init_key, key_length).
c
c  Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
c  All rights reserved.                          
c  Copyright (C) 2005, Mutsuo Saito,
c  All rights reserved.                          
c
c  Redistribution and use in source and binary forms, with or without
c  modification, are permitted provided that the following conditions
c  are met:
c
c    1. Redistributions of source code must retain the above copyright
c       notice, this list of conditions and the following disclaimer.
c
c    2. Redistributions in binary form must reproduce the above copyright
c       notice, this list of conditions and the following disclaimer in the
c       documentation and/or other materials provided with the distribution.
c
c    3. The names of its contributors may not be used to endorse or promote 
c       products derived from this software without specific prior written 
c       permission.
c
c  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
c  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
c  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
c  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
c  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
c  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
c  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
c  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
c  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
c  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
c  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
c
c
c  Any feedback is very welcome.
c  http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
c  email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
c
c-----------------------------------------------------------------------
c
c     ---------- initialize routines ----------
c  subroutine init_genrand(seed): initialize with a seed
c  subroutine init_by_array(init_key,key_length): initialize by an array
c
c     ---------- generate functions ----------
c  integer function genrand_int32(): signed 32-bit integer
c  integer function genrand_int31(): unsigned 31-bit integer
c  double precision function genrand_real1(): [0,1] with 32-bit resolution
c  double precision function genrand_real2(): [0,1) with 32-bit resolution
c  double precision function genrand_real3(): (0,1) with 32-bit resolution
c  double precision function genrand_res53(): (0,1) with 53-bit resolution
c
c  This program uses the following non-standard intrinsics.
c    ishft(i,n): If n>0, shifts bits in i by n positions to left.
c                If n<0, shifts bits in i by n positions to right.
c    iand (i,j): Performs logical AND on corresponding bits of i and j.
c    ior  (i,j): Performs inclusive OR on corresponding bits of i and j.
c    ieor (i,j): Performs exclusive OR on corresponding bits of i and j.
c
c-----------------------------------------------------------------------
c     initialize mt(0:N-1) with a seed
c-----------------------------------------------------------------------
      subroutine init_genrand(s)
      integer s
      integer N
      integer DONE
      integer ALLBIT_MASK
      parameter (N=624)
      parameter (DONE=123456789)
      integer mti,initialized
      integer mt(0:N-1)
      common /mt_state1/ mti,initialized
      common /mt_state2/ mt
      common /mt_mask1/ ALLBIT_MASK
c
      call mt_initln
      mt(0)=iand(s,ALLBIT_MASK)
      do 100 mti=1,N-1
        mt(mti)=1812433253*
     &          ieor(mt(mti-1),ishft(mt(mti-1),-30))+mti
        mt(mti)=iand(mt(mti),ALLBIT_MASK)
  100 continue
      initialized=DONE
c
      return
      end
c-----------------------------------------------------------------------
c     initialize by an array with array-length
c     init_key is the array for initializing keys
c     key_length is its length
c-----------------------------------------------------------------------
      subroutine init_by_array(init_key,key_length)
      integer init_key(0:*)
      integer key_length
      integer N
      integer ALLBIT_MASK
      integer TOPBIT_MASK
      parameter (N=624)
      integer i,j,k
      integer mt(0:N-1)
      common /mt_state2/ mt
      common /mt_mask1/ ALLBIT_MASK
      common /mt_mask2/ TOPBIT_MASK
c
      call init_genrand(19650218)
      i=1
      j=0
      do 100 k=max(N,key_length),1,-1
        mt(i)=ieor(mt(i),ieor(mt(i-1),ishft(mt(i-1),-30))*1664525)
     &           +init_key(j)+j
        mt(i)=iand(mt(i),ALLBIT_MASK)
        i=i+1
        j=j+1
        if(i.ge.N)then
          mt(0)=mt(N-1)
          i=1
        endif
        if(j.ge.key_length)then
          j=0
        endif
  100 continue
      do 200 k=N-1,1,-1
        mt(i)=ieor(mt(i),ieor(mt(i-1),ishft(mt(i-1),-30))*1566083941)-i
        mt(i)=iand(mt(i),ALLBIT_MASK)
        i=i+1
        if(i.ge.N)then
          mt(0)=mt(N-1)
          i=1
        endif
  200 continue
      mt(0)=TOPBIT_MASK
c
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on [0,0xffffffff]-interval
c-----------------------------------------------------------------------
      function genrand_int32()
      integer genrand_int32
      integer N,M
      integer DONE
      integer UPPER_MASK,LOWER_MASK,MATRIX_A
      integer T1_MASK,T2_MASK
      parameter (N=624)
      parameter (M=397)
      parameter (DONE=123456789)
      integer mti,initialized
      integer mt(0:N-1)
      integer y,kk
      integer mag01(0:1)
      common /mt_state1/ mti,initialized
      common /mt_state2/ mt
      common /mt_mask3/ UPPER_MASK,LOWER_MASK,MATRIX_A,T1_MASK,T2_MASK
      common /mt_mag01/ mag01
c
      if(initialized.ne.DONE)then
        call init_genrand(21641)
      endif
c
      if(mti.ge.N)then
        do 100 kk=0,N-M-1
          y=ior(iand(mt(kk),UPPER_MASK),iand(mt(kk+1),LOWER_MASK))
          mt(kk)=ieor(ieor(mt(kk+M),ishft(y,-1)),mag01(iand(y,1)))
  100   continue
        do 200 kk=N-M,N-1-1
          y=ior(iand(mt(kk),UPPER_MASK),iand(mt(kk+1),LOWER_MASK))
          mt(kk)=ieor(ieor(mt(kk+(M-N)),ishft(y,-1)),mag01(iand(y,1)))
  200   continue
        y=ior(iand(mt(N-1),UPPER_MASK),iand(mt(0),LOWER_MASK))
        mt(kk)=ieor(ieor(mt(M-1),ishft(y,-1)),mag01(iand(y,1)))
        mti=0
      endif
c
      y=mt(mti)
      mti=mti+1
c
      y=ieor(y,ishft(y,-11))
      y=ieor(y,iand(ishft(y,7),T1_MASK))
      y=ieor(y,iand(ishft(y,15),T2_MASK))
      y=ieor(y,ishft(y,-18))
c
      genrand_int32=y
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on [0,0x7fffffff]-interval
c-----------------------------------------------------------------------
      function genrand_int31()
      integer genrand_int31
      integer genrand_int32
      genrand_int31=int(ishft(genrand_int32(),-1))
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on [0,1]-real-interval
c-----------------------------------------------------------------------
      function genrand_real1()
      double precision genrand_real1,r
      integer genrand_int32
      r=dble(genrand_int32())
      if(r.lt.0.d0)r=r+2.d0**32
      genrand_real1=r/4294967295.d0
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on [0,1)-real-interval
c-----------------------------------------------------------------------
      function genrand_real2()
      double precision genrand_real2,r
      integer genrand_int32
      r=dble(genrand_int32())
      if(r.lt.0.d0)r=r+2.d0**32
      genrand_real2=r/4294967296.d0
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on (0,1)-real-interval
c-----------------------------------------------------------------------
      function genrand_real3()
      double precision genrand_real3,r
      integer genrand_int32
      r=dble(genrand_int32())
      if(r.lt.0.d0)r=r+2.d0**32
      genrand_real3=(r+0.5d0)/4294967296.d0
      return
      end
c-----------------------------------------------------------------------
c     generates a random number on [0,1) with 53-bit resolution
c-----------------------------------------------------------------------
      function genrand_res53()
      double precision genrand_res53
      integer genrand_int32
      double precision a,b
      a=dble(ishft(genrand_int32(),-5))
      b=dble(ishft(genrand_int32(),-6))
      if(a.lt.0.d0)a=a+2.d0**32
      if(b.lt.0.d0)b=b+2.d0**32
      genrand_res53=(a*67108864.d0+b)/9007199254740992.d0
      return
      end
c-----------------------------------------------------------------------
c     initialize large number (over 32-bit constant number)
c-----------------------------------------------------------------------
      subroutine mt_initln
      integer ALLBIT_MASK
      integer TOPBIT_MASK
      integer UPPER_MASK,LOWER_MASK,MATRIX_A,T1_MASK,T2_MASK
      integer mag01(0:1)
      common /mt_mask1/ ALLBIT_MASK
      common /mt_mask2/ TOPBIT_MASK
      common /mt_mask3/ UPPER_MASK,LOWER_MASK,MATRIX_A,T1_MASK,T2_MASK
      common /mt_mag01/ mag01
CC    TOPBIT_MASK = Z'80000000'
CC    ALLBIT_MASK = Z'ffffffff'
CC    UPPER_MASK  = Z'80000000'
CC    LOWER_MASK  = Z'7fffffff'
CC    MATRIX_A    = Z'9908b0df'
CC    T1_MASK     = Z'9d2c5680'
CC    T2_MASK     = Z'efc60000'
      TOPBIT_MASK=1073741824
      TOPBIT_MASK=ishft(TOPBIT_MASK,1)
      ALLBIT_MASK=2147483647
      ALLBIT_MASK=ior(ALLBIT_MASK,TOPBIT_MASK)
      UPPER_MASK=TOPBIT_MASK
      LOWER_MASK=2147483647
      MATRIX_A=419999967
      MATRIX_A=ior(MATRIX_A,TOPBIT_MASK)
      T1_MASK=489444992
      T1_MASK=ior(T1_MASK,TOPBIT_MASK)
      T2_MASK=1875247104
      T2_MASK=ior(T2_MASK,TOPBIT_MASK)
      mag01(0)=0
      mag01(1)=MATRIX_A
      return
      end
c=======================================================================
      real function bran()
c
c  This function returns a pseudo-random number for each invocation.
c  It is a Fortran version of the "Integer Version 2" minimal 
c  standard number generator from the Pascal code in:
c
c     Park, Steven K. and Miller, Keith W., "Random Number Generators: 
c     Good Ones are Hard to Find", Communications of the ACM, 
c     October, 1988.
c
      parameter (mp=16807, mod=2147483647, kf=127773, md=2836)
      common/rseed/iseed,mseed
      integer i1, it, nx
      save nx,s
      data ifrst/0/
      if (ifrst.eq.0) then
        ifrst = 1
        nx = mseed
        s=1./real(mod)
      endif
      i1 = nx/kf
      it = mp*(nx-i1*kf) - md*i1
      if (it.gt.0) then
        nx = it
      else
        nx = it + mod
      endif
      bran = s*real(nx)
      return
      end
c=======================================================================
      function gauss()
c
c    returns a Gaussian variable 
c
 1    v1=2*bran()-1
      v2=2*bran()-1
      r=v1**2+v2**2
      if(r.ge.1) go to 1
      gauss = v1*sqrt(-2.*alog(r)/r)
      return
      end
